// Copyright (c) 2008-2011 Mighty Heave Software LLC.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#include "chtest.h"
#ifdef _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>
#endif

bool glr_isspace(codepoint c)
{
    if( c >= 9 && c <= 13 ) // CHARACTER TABULATION -> CARRIAGE RETURN (CR)
        return true;
    if( c == 32 ) // SPACE
        return true;
    if( c == 133 ) // NEXT LINE (NEL)
        return true;
    if( c == 160 ) // NO-BREAK SPACE
        return true;
    if( c == 5760 ) // OGHAM SPACE MARK
        return true;
    if( c == 6158 ) // MONGOLIAN VOWEL SEPARATOR
        return true;
    if( c >= 8192 && c <= 8202 ) // EN QUAD -> HAIR SPACE
        return true;
    if( c >= 8232 && c <= 8233 ) // LINE SEPARATOR -> PARAGRAPH SEPARATOR
        return true;
    if( c == 8239 ) // NARROW NO-BREAK SPACE
        return true;
    if( c == 8287 ) // MEDIUM MATHEMATICAL SPACE
        return true;
    if( c == 12288 ) // IDEOGRAPHIC SPACE
        return true;
    return false;
}

bool glr_isdigit(codepoint c)
{
    if( c >= 48 && c <= 57 ) // DIGIT ZERO -> DIGIT NINE
        return true;
    if( c >= 1632 && c <= 1641 ) // ARABIC-INDIC DIGIT ZERO -> ARABIC-INDIC DIGIT NINE
        return true;
    if( c >= 1776 && c <= 1785 ) // EXTENDED ARABIC-INDIC DIGIT ZERO -> EXTENDED ARABIC-INDIC DIGIT NINE
        return true;
    if( c >= 1984 && c <= 1993 ) // NKO DIGIT ZERO -> NKO DIGIT NINE
        return true;
    if( c >= 2406 && c <= 2415 ) // DEVANAGARI DIGIT ZERO -> DEVANAGARI DIGIT NINE
        return true;
    if( c >= 2534 && c <= 2543 ) // BENGALI DIGIT ZERO -> BENGALI DIGIT NINE
        return true;
    if( c >= 2662 && c <= 2671 ) // GURMUKHI DIGIT ZERO -> GURMUKHI DIGIT NINE
        return true;
    if( c >= 2790 && c <= 2799 ) // GUJARATI DIGIT ZERO -> GUJARATI DIGIT NINE
        return true;
    if( c >= 2918 && c <= 2927 ) // ORIYA DIGIT ZERO -> ORIYA DIGIT NINE
        return true;
    if( c >= 3046 && c <= 3055 ) // TAMIL DIGIT ZERO -> TAMIL DIGIT NINE
        return true;
    if( c >= 3174 && c <= 3183 ) // TELUGU DIGIT ZERO -> TELUGU DIGIT NINE
        return true;
    if( c >= 3302 && c <= 3311 ) // KANNADA DIGIT ZERO -> KANNADA DIGIT NINE
        return true;
    if( c >= 3430 && c <= 3439 ) // MALAYALAM DIGIT ZERO -> MALAYALAM DIGIT NINE
        return true;
    if( c >= 3664 && c <= 3673 ) // THAI DIGIT ZERO -> THAI DIGIT NINE
        return true;
    if( c >= 3792 && c <= 3801 ) // LAO DIGIT ZERO -> LAO DIGIT NINE
        return true;
    if( c >= 3872 && c <= 3881 ) // TIBETAN DIGIT ZERO -> TIBETAN DIGIT NINE
        return true;
    if( c >= 4160 && c <= 4169 ) // MYANMAR DIGIT ZERO -> MYANMAR DIGIT NINE
        return true;
    if( c >= 4240 && c <= 4249 ) // MYANMAR SHAN DIGIT ZERO -> MYANMAR SHAN DIGIT NINE
        return true;
    if( c >= 6112 && c <= 6121 ) // KHMER DIGIT ZERO -> KHMER DIGIT NINE
        return true;
    if( c >= 6160 && c <= 6169 ) // MONGOLIAN DIGIT ZERO -> MONGOLIAN DIGIT NINE
        return true;
    if( c >= 6470 && c <= 6479 ) // LIMBU DIGIT ZERO -> LIMBU DIGIT NINE
        return true;
    if( c >= 6608 && c <= 6617 ) // NEW TAI LUE DIGIT ZERO -> NEW TAI LUE DIGIT NINE
        return true;
    if( c >= 6992 && c <= 7001 ) // BALINESE DIGIT ZERO -> BALINESE DIGIT NINE
        return true;
    if( c >= 7088 && c <= 7097 ) // SUNDANESE DIGIT ZERO -> SUNDANESE DIGIT NINE
        return true;
    if( c >= 7232 && c <= 7241 ) // LEPCHA DIGIT ZERO -> LEPCHA DIGIT NINE
        return true;
    if( c >= 7248 && c <= 7257 ) // OL CHIKI DIGIT ZERO -> OL CHIKI DIGIT NINE
        return true;
    if( c >= 42528 && c <= 42537 ) // VAI DIGIT ZERO -> VAI DIGIT NINE
        return true;
    if( c >= 43216 && c <= 43225 ) // SAURASHTRA DIGIT ZERO -> SAURASHTRA DIGIT NINE
        return true;
    if( c >= 43264 && c <= 43273 ) // KAYAH LI DIGIT ZERO -> KAYAH LI DIGIT NINE
        return true;
    if( c >= 43600 && c <= 43609 ) // CHAM DIGIT ZERO -> CHAM DIGIT NINE
        return true;
    if( c >= 65296 && c <= 65305 ) // FULLWIDTH DIGIT ZERO -> FULLWIDTH DIGIT NINE
        return true;
    if( c >= 66720 && c <= 66729 ) // OSMANYA DIGIT ZERO -> OSMANYA DIGIT NINE
        return true;
    if( c >= 120782 && c <= 120831 ) // MATHEMATICAL BOLD DIGIT ZERO -> MATHEMATICAL MONOSPACE DIGIT NINE
        return true;
    return false;
}

int glr_digitvalue(codepoint c)
{
    if( c >= 48 && c <= 57 ) // DIGIT ZERO -> DIGIT NINE
        return (c-48);
    if( c >= 1632 && c <= 1641 ) // ARABIC-INDIC DIGIT ZERO -> ARABIC-INDIC DIGIT NINE
        return (c-1632);
    if( c >= 1776 && c <= 1785 ) // EXTENDED ARABIC-INDIC DIGIT ZERO -> EXTENDED ARABIC-INDIC DIGIT NINE
        return (c-1776);
    if( c >= 1984 && c <= 1993 ) // NKO DIGIT ZERO -> NKO DIGIT NINE
        return (c-1984);
    if( c >= 2406 && c <= 2415 ) // DEVANAGARI DIGIT ZERO -> DEVANAGARI DIGIT NINE
        return (c-2406);
    if( c >= 2534 && c <= 2543 ) // BENGALI DIGIT ZERO -> BENGALI DIGIT NINE
        return (c-2534);
    if( c >= 2662 && c <= 2671 ) // GURMUKHI DIGIT ZERO -> GURMUKHI DIGIT NINE
        return (c-2662);
    if( c >= 2790 && c <= 2799 ) // GUJARATI DIGIT ZERO -> GUJARATI DIGIT NINE
        return (c-2790);
    if( c >= 2918 && c <= 2927 ) // ORIYA DIGIT ZERO -> ORIYA DIGIT NINE
        return (c-2918);
    if( c >= 3046 && c <= 3055 ) // TAMIL DIGIT ZERO -> TAMIL DIGIT NINE
        return (c-3046);
    if( c >= 3174 && c <= 3183 ) // TELUGU DIGIT ZERO -> TELUGU DIGIT NINE
        return (c-3174);
    if( c >= 3302 && c <= 3311 ) // KANNADA DIGIT ZERO -> KANNADA DIGIT NINE
        return (c-3302);
    if( c >= 3430 && c <= 3439 ) // MALAYALAM DIGIT ZERO -> MALAYALAM DIGIT NINE
        return (c-3430);
    if( c >= 3664 && c <= 3673 ) // THAI DIGIT ZERO -> THAI DIGIT NINE
        return (c-3664);
    if( c >= 3792 && c <= 3801 ) // LAO DIGIT ZERO -> LAO DIGIT NINE
        return (c-3792);
    if( c >= 3872 && c <= 3881 ) // TIBETAN DIGIT ZERO -> TIBETAN DIGIT NINE
        return (c-3872);
    if( c >= 4160 && c <= 4169 ) // MYANMAR DIGIT ZERO -> MYANMAR DIGIT NINE
        return (c-4160);
    if( c >= 4240 && c <= 4249 ) // MYANMAR SHAN DIGIT ZERO -> MYANMAR SHAN DIGIT NINE
        return (c-4240);
    if( c >= 6112 && c <= 6121 ) // KHMER DIGIT ZERO -> KHMER DIGIT NINE
        return (c-6112);
    if( c >= 6160 && c <= 6169 ) // MONGOLIAN DIGIT ZERO -> MONGOLIAN DIGIT NINE
        return (c-6160);
    if( c >= 6470 && c <= 6479 ) // LIMBU DIGIT ZERO -> LIMBU DIGIT NINE
        return (c-6470);
    if( c >= 6608 && c <= 6617 ) // NEW TAI LUE DIGIT ZERO -> NEW TAI LUE DIGIT NINE
        return (c-6608);
    if( c >= 6992 && c <= 7001 ) // BALINESE DIGIT ZERO -> BALINESE DIGIT NINE
        return (c-6992);
    if( c >= 7088 && c <= 7097 ) // SUNDANESE DIGIT ZERO -> SUNDANESE DIGIT NINE
        return (c-7088);
    if( c >= 7232 && c <= 7241 ) // LEPCHA DIGIT ZERO -> LEPCHA DIGIT NINE
        return (c-7232);
    if( c >= 7248 && c <= 7257 ) // OL CHIKI DIGIT ZERO -> OL CHIKI DIGIT NINE
        return (c-7248);
    if( c >= 42528 && c <= 42537 ) // VAI DIGIT ZERO -> VAI DIGIT NINE
        return (c-42528);
    if( c >= 43216 && c <= 43225 ) // SAURASHTRA DIGIT ZERO -> SAURASHTRA DIGIT NINE
        return (c-43216);
    if( c >= 43264 && c <= 43273 ) // KAYAH LI DIGIT ZERO -> KAYAH LI DIGIT NINE
        return (c-43264);
    if( c >= 43600 && c <= 43609 ) // CHAM DIGIT ZERO -> CHAM DIGIT NINE
        return (c-43600);
    if( c >= 65296 && c <= 65305 ) // FULLWIDTH DIGIT ZERO -> FULLWIDTH DIGIT NINE
        return (c-65296);
    if( c >= 66720 && c <= 66729 ) // OSMANYA DIGIT ZERO -> OSMANYA DIGIT NINE
        return (c-66720);
    if( c >= 120782 && c <= 120791 ) // MATHEMATICAL BOLD DIGIT ZERO -> MATHEMATICAL BOLD DIGIT NINE
        return (c-120782);
    if( c >= 120792 && c <= 120801 ) // MATHEMATICAL DOUBLE-STRUCK DIGIT ZERO -> MATHEMATICAL DOUBLE-STRUCK DIGIT NINE
        return (c-120792);
    if( c >= 120802 && c <= 120811 ) // MATHEMATICAL SANS-SERIF DIGIT ZERO -> MATHEMATICAL SANS-SERIF DIGIT NINE
        return (c-120802);
    if( c >= 120812 && c <= 120821 ) // MATHEMATICAL SANS-SERIF BOLD DIGIT ZERO -> MATHEMATICAL SANS-SERIF BOLD DIGIT NINE
        return (c-120812);
    if( c >= 120822 && c <= 120831 ) // MATHEMATICAL MONOSPACE DIGIT ZERO -> MATHEMATICAL MONOSPACE DIGIT NINE
        return (c-120822);
    return -1;
}

bool glr_isxdigit(codepoint c)
{
    if( c >= 48 && c <= 57 ) // DIGIT ZERO -> DIGIT NINE
        return true;
    if( c >= 65 && c <= 70 ) // LATIN CAPITAL LETTER A -> LATIN CAPITAL LETTER F
        return true;
    if( c >= 97 && c <= 102 ) // LATIN SMALL LETTER A -> LATIN SMALL LETTER F
        return true;
    if( c >= 65296 && c <= 65305 ) // FULLWIDTH DIGIT ZERO -> FULLWIDTH DIGIT NINE
        return true;
    if( c >= 65313 && c <= 65318 ) // FULLWIDTH LATIN CAPITAL LETTER A -> FULLWIDTH LATIN CAPITAL LETTER F
        return true;
    if( c >= 65345 && c <= 65350 ) // FULLWIDTH LATIN SMALL LETTER A -> FULLWIDTH LATIN SMALL LETTER F
        return true;
    return false;
}

int glr_xdigitvalue(codepoint c)
{
    if( c >= 48 && c <= 57 ) // DIGIT ZERO -> DIGIT NINE
        return (c-48);
    if( c >= 65 && c <= 70 ) // LATIN CAPITAL LETTER A -> LATIN CAPITAL LETTER F
        return 10+(c-65);
    if( c >= 97 && c <= 102 ) // LATIN SMALL LETTER A -> LATIN SMALL LETTER F
        return 10+(c-97);
    if( c >= 65296 && c <= 65305 ) // FULLWIDTH DIGIT ZERO -> FULLWIDTH DIGIT NINE
        return (c-65296);
    if( c >= 65313 && c <= 65318 ) // FULLWIDTH LATIN CAPITAL LETTER A -> FULLWIDTH LATIN CAPITAL LETTER F
        return 10+(c-65313);
    if( c >= 65345 && c <= 65350 ) // FULLWIDTH LATIN SMALL LETTER A -> FULLWIDTH LATIN SMALL LETTER F
        return 10+(c-65345);
    return -1;
}

codepoint glr_toupper(codepoint c)
{
    if( c >= 97 && c <= 122 ) // LATIN SMALL LETTER A -> LATIN SMALL LETTER Z
        return 65+(c-97);
    if( c == 181 ) // MICRO SIGN
        return 924;
    if( c >= 224 && c <= 246 ) // LATIN SMALL LETTER A WITH GRAVE -> LATIN SMALL LETTER O WITH DIAERESIS
        return 192+(c-224);
    if( c >= 248 && c <= 254 ) // LATIN SMALL LETTER O WITH STROKE -> LATIN SMALL LETTER THORN
        return 216+(c-248);
    if( c == 255 ) // LATIN SMALL LETTER Y WITH DIAERESIS
        return 376;
    if( c == 257 ) // LATIN SMALL LETTER A WITH MACRON
        return 256;
    if( c == 259 ) // LATIN SMALL LETTER A WITH BREVE
        return 258;
    if( c == 261 ) // LATIN SMALL LETTER A WITH OGONEK
        return 260;
    if( c == 263 ) // LATIN SMALL LETTER C WITH ACUTE
        return 262;
    if( c == 265 ) // LATIN SMALL LETTER C WITH CIRCUMFLEX
        return 264;
    if( c == 267 ) // LATIN SMALL LETTER C WITH DOT ABOVE
        return 266;
    if( c == 269 ) // LATIN SMALL LETTER C WITH CARON
        return 268;
    if( c == 271 ) // LATIN SMALL LETTER D WITH CARON
        return 270;
    if( c == 273 ) // LATIN SMALL LETTER D WITH STROKE
        return 272;
    if( c == 275 ) // LATIN SMALL LETTER E WITH MACRON
        return 274;
    if( c == 277 ) // LATIN SMALL LETTER E WITH BREVE
        return 276;
    if( c == 279 ) // LATIN SMALL LETTER E WITH DOT ABOVE
        return 278;
    if( c == 281 ) // LATIN SMALL LETTER E WITH OGONEK
        return 280;
    if( c == 283 ) // LATIN SMALL LETTER E WITH CARON
        return 282;
    if( c == 285 ) // LATIN SMALL LETTER G WITH CIRCUMFLEX
        return 284;
    if( c == 287 ) // LATIN SMALL LETTER G WITH BREVE
        return 286;
    if( c == 289 ) // LATIN SMALL LETTER G WITH DOT ABOVE
        return 288;
    if( c == 291 ) // LATIN SMALL LETTER G WITH CEDILLA
        return 290;
    if( c == 293 ) // LATIN SMALL LETTER H WITH CIRCUMFLEX
        return 292;
    if( c == 295 ) // LATIN SMALL LETTER H WITH STROKE
        return 294;
    if( c == 297 ) // LATIN SMALL LETTER I WITH TILDE
        return 296;
    if( c == 299 ) // LATIN SMALL LETTER I WITH MACRON
        return 298;
    if( c == 301 ) // LATIN SMALL LETTER I WITH BREVE
        return 300;
    if( c == 303 ) // LATIN SMALL LETTER I WITH OGONEK
        return 302;
    if( c == 305 ) // LATIN SMALL LETTER DOTLESS I
        return 73;
    if( c == 307 ) // LATIN SMALL LIGATURE IJ
        return 306;
    if( c == 309 ) // LATIN SMALL LETTER J WITH CIRCUMFLEX
        return 308;
    if( c == 311 ) // LATIN SMALL LETTER K WITH CEDILLA
        return 310;
    if( c == 314 ) // LATIN SMALL LETTER L WITH ACUTE
        return 313;
    if( c == 316 ) // LATIN SMALL LETTER L WITH CEDILLA
        return 315;
    if( c == 318 ) // LATIN SMALL LETTER L WITH CARON
        return 317;
    if( c == 320 ) // LATIN SMALL LETTER L WITH MIDDLE DOT
        return 319;
    if( c == 322 ) // LATIN SMALL LETTER L WITH STROKE
        return 321;
    if( c == 324 ) // LATIN SMALL LETTER N WITH ACUTE
        return 323;
    if( c == 326 ) // LATIN SMALL LETTER N WITH CEDILLA
        return 325;
    if( c == 328 ) // LATIN SMALL LETTER N WITH CARON
        return 327;
    if( c == 331 ) // LATIN SMALL LETTER ENG
        return 330;
    if( c == 333 ) // LATIN SMALL LETTER O WITH MACRON
        return 332;
    if( c == 335 ) // LATIN SMALL LETTER O WITH BREVE
        return 334;
    if( c == 337 ) // LATIN SMALL LETTER O WITH DOUBLE ACUTE
        return 336;
    if( c == 339 ) // LATIN SMALL LIGATURE OE
        return 338;
    if( c == 341 ) // LATIN SMALL LETTER R WITH ACUTE
        return 340;
    if( c == 343 ) // LATIN SMALL LETTER R WITH CEDILLA
        return 342;
    if( c == 345 ) // LATIN SMALL LETTER R WITH CARON
        return 344;
    if( c == 347 ) // LATIN SMALL LETTER S WITH ACUTE
        return 346;
    if( c == 349 ) // LATIN SMALL LETTER S WITH CIRCUMFLEX
        return 348;
    if( c == 351 ) // LATIN SMALL LETTER S WITH CEDILLA
        return 350;
    if( c == 353 ) // LATIN SMALL LETTER S WITH CARON
        return 352;
    if( c == 355 ) // LATIN SMALL LETTER T WITH CEDILLA
        return 354;
    if( c == 357 ) // LATIN SMALL LETTER T WITH CARON
        return 356;
    if( c == 359 ) // LATIN SMALL LETTER T WITH STROKE
        return 358;
    if( c == 361 ) // LATIN SMALL LETTER U WITH TILDE
        return 360;
    if( c == 363 ) // LATIN SMALL LETTER U WITH MACRON
        return 362;
    if( c == 365 ) // LATIN SMALL LETTER U WITH BREVE
        return 364;
    if( c == 367 ) // LATIN SMALL LETTER U WITH RING ABOVE
        return 366;
    if( c == 369 ) // LATIN SMALL LETTER U WITH DOUBLE ACUTE
        return 368;
    if( c == 371 ) // LATIN SMALL LETTER U WITH OGONEK
        return 370;
    if( c == 373 ) // LATIN SMALL LETTER W WITH CIRCUMFLEX
        return 372;
    if( c == 375 ) // LATIN SMALL LETTER Y WITH CIRCUMFLEX
        return 374;
    if( c == 378 ) // LATIN SMALL LETTER Z WITH ACUTE
        return 377;
    if( c == 380 ) // LATIN SMALL LETTER Z WITH DOT ABOVE
        return 379;
    if( c == 382 ) // LATIN SMALL LETTER Z WITH CARON
        return 381;
    if( c == 383 ) // LATIN SMALL LETTER LONG S
        return 83;
    if( c == 384 ) // LATIN SMALL LETTER B WITH STROKE
        return 579;
    if( c == 387 ) // LATIN SMALL LETTER B WITH TOPBAR
        return 386;
    if( c == 389 ) // LATIN SMALL LETTER TONE SIX
        return 388;
    if( c == 392 ) // LATIN SMALL LETTER C WITH HOOK
        return 391;
    if( c == 396 ) // LATIN SMALL LETTER D WITH TOPBAR
        return 395;
    if( c == 402 ) // LATIN SMALL LETTER F WITH HOOK
        return 401;
    if( c == 405 ) // LATIN SMALL LETTER HV
        return 502;
    if( c == 409 ) // LATIN SMALL LETTER K WITH HOOK
        return 408;
    if( c == 410 ) // LATIN SMALL LETTER L WITH BAR
        return 573;
    if( c == 414 ) // LATIN SMALL LETTER N WITH LONG RIGHT LEG
        return 544;
    if( c == 417 ) // LATIN SMALL LETTER O WITH HORN
        return 416;
    if( c == 419 ) // LATIN SMALL LETTER OI
        return 418;
    if( c == 421 ) // LATIN SMALL LETTER P WITH HOOK
        return 420;
    if( c == 424 ) // LATIN SMALL LETTER TONE TWO
        return 423;
    if( c == 429 ) // LATIN SMALL LETTER T WITH HOOK
        return 428;
    if( c == 432 ) // LATIN SMALL LETTER U WITH HORN
        return 431;
    if( c == 436 ) // LATIN SMALL LETTER Y WITH HOOK
        return 435;
    if( c == 438 ) // LATIN SMALL LETTER Z WITH STROKE
        return 437;
    if( c == 441 ) // LATIN SMALL LETTER EZH REVERSED
        return 440;
    if( c == 445 ) // LATIN SMALL LETTER TONE FIVE
        return 444;
    if( c == 447 ) // LATIN LETTER WYNN
        return 503;
    if( c == 454 ) // LATIN SMALL LETTER DZ WITH CARON
        return 452;
    if( c == 457 ) // LATIN SMALL LETTER LJ
        return 455;
    if( c == 460 ) // LATIN SMALL LETTER NJ
        return 458;
    if( c == 462 ) // LATIN SMALL LETTER A WITH CARON
        return 461;
    if( c == 464 ) // LATIN SMALL LETTER I WITH CARON
        return 463;
    if( c == 466 ) // LATIN SMALL LETTER O WITH CARON
        return 465;
    if( c == 468 ) // LATIN SMALL LETTER U WITH CARON
        return 467;
    if( c == 470 ) // LATIN SMALL LETTER U WITH DIAERESIS AND MACRON
        return 469;
    if( c == 472 ) // LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE
        return 471;
    if( c == 474 ) // LATIN SMALL LETTER U WITH DIAERESIS AND CARON
        return 473;
    if( c == 476 ) // LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE
        return 475;
    if( c == 477 ) // LATIN SMALL LETTER TURNED E
        return 398;
    if( c == 479 ) // LATIN SMALL LETTER A WITH DIAERESIS AND MACRON
        return 478;
    if( c == 481 ) // LATIN SMALL LETTER A WITH DOT ABOVE AND MACRON
        return 480;
    if( c == 483 ) // LATIN SMALL LETTER AE WITH MACRON
        return 482;
    if( c == 485 ) // LATIN SMALL LETTER G WITH STROKE
        return 484;
    if( c == 487 ) // LATIN SMALL LETTER G WITH CARON
        return 486;
    if( c == 489 ) // LATIN SMALL LETTER K WITH CARON
        return 488;
    if( c == 491 ) // LATIN SMALL LETTER O WITH OGONEK
        return 490;
    if( c == 493 ) // LATIN SMALL LETTER O WITH OGONEK AND MACRON
        return 492;
    if( c == 495 ) // LATIN SMALL LETTER EZH WITH CARON
        return 494;
    if( c == 499 ) // LATIN SMALL LETTER DZ
        return 497;
    if( c == 501 ) // LATIN SMALL LETTER G WITH ACUTE
        return 500;
    if( c == 505 ) // LATIN SMALL LETTER N WITH GRAVE
        return 504;
    if( c == 507 ) // LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE
        return 506;
    if( c == 509 ) // LATIN SMALL LETTER AE WITH ACUTE
        return 508;
    if( c == 511 ) // LATIN SMALL LETTER O WITH STROKE AND ACUTE
        return 510;
    if( c == 513 ) // LATIN SMALL LETTER A WITH DOUBLE GRAVE
        return 512;
    if( c == 515 ) // LATIN SMALL LETTER A WITH INVERTED BREVE
        return 514;
    if( c == 517 ) // LATIN SMALL LETTER E WITH DOUBLE GRAVE
        return 516;
    if( c == 519 ) // LATIN SMALL LETTER E WITH INVERTED BREVE
        return 518;
    if( c == 521 ) // LATIN SMALL LETTER I WITH DOUBLE GRAVE
        return 520;
    if( c == 523 ) // LATIN SMALL LETTER I WITH INVERTED BREVE
        return 522;
    if( c == 525 ) // LATIN SMALL LETTER O WITH DOUBLE GRAVE
        return 524;
    if( c == 527 ) // LATIN SMALL LETTER O WITH INVERTED BREVE
        return 526;
    if( c == 529 ) // LATIN SMALL LETTER R WITH DOUBLE GRAVE
        return 528;
    if( c == 531 ) // LATIN SMALL LETTER R WITH INVERTED BREVE
        return 530;
    if( c == 533 ) // LATIN SMALL LETTER U WITH DOUBLE GRAVE
        return 532;
    if( c == 535 ) // LATIN SMALL LETTER U WITH INVERTED BREVE
        return 534;
    if( c == 537 ) // LATIN SMALL LETTER S WITH COMMA BELOW
        return 536;
    if( c == 539 ) // LATIN SMALL LETTER T WITH COMMA BELOW
        return 538;
    if( c == 541 ) // LATIN SMALL LETTER YOGH
        return 540;
    if( c == 543 ) // LATIN SMALL LETTER H WITH CARON
        return 542;
    if( c == 547 ) // LATIN SMALL LETTER OU
        return 546;
    if( c == 549 ) // LATIN SMALL LETTER Z WITH HOOK
        return 548;
    if( c == 551 ) // LATIN SMALL LETTER A WITH DOT ABOVE
        return 550;
    if( c == 553 ) // LATIN SMALL LETTER E WITH CEDILLA
        return 552;
    if( c == 555 ) // LATIN SMALL LETTER O WITH DIAERESIS AND MACRON
        return 554;
    if( c == 557 ) // LATIN SMALL LETTER O WITH TILDE AND MACRON
        return 556;
    if( c == 559 ) // LATIN SMALL LETTER O WITH DOT ABOVE
        return 558;
    if( c == 561 ) // LATIN SMALL LETTER O WITH DOT ABOVE AND MACRON
        return 560;
    if( c == 563 ) // LATIN SMALL LETTER Y WITH MACRON
        return 562;
    if( c == 572 ) // LATIN SMALL LETTER C WITH STROKE
        return 571;
    if( c == 578 ) // LATIN SMALL LETTER GLOTTAL STOP
        return 577;
    if( c == 583 ) // LATIN SMALL LETTER E WITH STROKE
        return 582;
    if( c == 585 ) // LATIN SMALL LETTER J WITH STROKE
        return 584;
    if( c == 587 ) // LATIN SMALL LETTER Q WITH HOOK TAIL
        return 586;
    if( c == 589 ) // LATIN SMALL LETTER R WITH STROKE
        return 588;
    if( c == 591 ) // LATIN SMALL LETTER Y WITH STROKE
        return 590;
    if( c == 592 ) // LATIN SMALL LETTER TURNED A
        return 11375;
    if( c == 593 ) // LATIN SMALL LETTER ALPHA
        return 11373;
    if( c == 595 ) // LATIN SMALL LETTER B WITH HOOK
        return 385;
    if( c == 596 ) // LATIN SMALL LETTER OPEN O
        return 390;
    if( c >= 598 && c <= 599 ) // LATIN SMALL LETTER D WITH TAIL -> LATIN SMALL LETTER D WITH HOOK
        return 393+(c-598);
    if( c == 601 ) // LATIN SMALL LETTER SCHWA
        return 399;
    if( c == 603 ) // LATIN SMALL LETTER OPEN E
        return 400;
    if( c == 608 ) // LATIN SMALL LETTER G WITH HOOK
        return 403;
    if( c == 611 ) // LATIN SMALL LETTER GAMMA
        return 404;
    if( c == 616 ) // LATIN SMALL LETTER I WITH STROKE
        return 407;
    if( c == 617 ) // LATIN SMALL LETTER IOTA
        return 406;
    if( c == 619 ) // LATIN SMALL LETTER L WITH MIDDLE TILDE
        return 11362;
    if( c == 623 ) // LATIN SMALL LETTER TURNED M
        return 412;
    if( c == 625 ) // LATIN SMALL LETTER M WITH HOOK
        return 11374;
    if( c == 626 ) // LATIN SMALL LETTER N WITH LEFT HOOK
        return 413;
    if( c == 629 ) // LATIN SMALL LETTER BARRED O
        return 415;
    if( c == 637 ) // LATIN SMALL LETTER R WITH TAIL
        return 11364;
    if( c == 640 ) // LATIN LETTER SMALL CAPITAL R
        return 422;
    if( c == 643 ) // LATIN SMALL LETTER ESH
        return 425;
    if( c == 648 ) // LATIN SMALL LETTER T WITH RETROFLEX HOOK
        return 430;
    if( c == 649 ) // LATIN SMALL LETTER U BAR
        return 580;
    if( c >= 650 && c <= 651 ) // LATIN SMALL LETTER UPSILON -> LATIN SMALL LETTER V WITH HOOK
        return 433+(c-650);
    if( c == 652 ) // LATIN SMALL LETTER TURNED V
        return 581;
    if( c == 658 ) // LATIN SMALL LETTER EZH
        return 439;
    if( c == 881 ) // GREEK SMALL LETTER HETA
        return 880;
    if( c == 883 ) // GREEK SMALL LETTER ARCHAIC SAMPI
        return 882;
    if( c == 887 ) // GREEK SMALL LETTER PAMPHYLIAN DIGAMMA
        return 886;
    if( c >= 891 && c <= 893 ) // GREEK SMALL REVERSED LUNATE SIGMA SYMBOL -> GREEK SMALL REVERSED DOTTED LUNATE SIGMA SYMBOL
        return 1021+(c-891);
    if( c == 940 ) // GREEK SMALL LETTER ALPHA WITH TONOS
        return 902;
    if( c >= 941 && c <= 943 ) // GREEK SMALL LETTER EPSILON WITH TONOS -> GREEK SMALL LETTER IOTA WITH TONOS
        return 904+(c-941);
    if( c >= 945 && c <= 961 ) // GREEK SMALL LETTER ALPHA -> GREEK SMALL LETTER RHO
        return 913+(c-945);
    if( c == 962 ) // GREEK SMALL LETTER FINAL SIGMA
        return 931;
    if( c >= 963 && c <= 971 ) // GREEK SMALL LETTER SIGMA -> GREEK SMALL LETTER UPSILON WITH DIALYTIKA
        return 931+(c-963);
    if( c == 972 ) // GREEK SMALL LETTER OMICRON WITH TONOS
        return 908;
    if( c >= 973 && c <= 974 ) // GREEK SMALL LETTER UPSILON WITH TONOS -> GREEK SMALL LETTER OMEGA WITH TONOS
        return 910+(c-973);
    if( c == 976 ) // GREEK BETA SYMBOL
        return 914;
    if( c == 977 ) // GREEK THETA SYMBOL
        return 920;
    if( c == 981 ) // GREEK PHI SYMBOL
        return 934;
    if( c == 982 ) // GREEK PI SYMBOL
        return 928;
    if( c == 983 ) // GREEK KAI SYMBOL
        return 975;
    if( c == 985 ) // GREEK SMALL LETTER ARCHAIC KOPPA
        return 984;
    if( c == 987 ) // GREEK SMALL LETTER STIGMA
        return 986;
    if( c == 989 ) // GREEK SMALL LETTER DIGAMMA
        return 988;
    if( c == 991 ) // GREEK SMALL LETTER KOPPA
        return 990;
    if( c == 993 ) // GREEK SMALL LETTER SAMPI
        return 992;
    if( c == 995 ) // COPTIC SMALL LETTER SHEI
        return 994;
    if( c == 997 ) // COPTIC SMALL LETTER FEI
        return 996;
    if( c == 999 ) // COPTIC SMALL LETTER KHEI
        return 998;
    if( c == 1001 ) // COPTIC SMALL LETTER HORI
        return 1000;
    if( c == 1003 ) // COPTIC SMALL LETTER GANGIA
        return 1002;
    if( c == 1005 ) // COPTIC SMALL LETTER SHIMA
        return 1004;
    if( c == 1007 ) // COPTIC SMALL LETTER DEI
        return 1006;
    if( c == 1008 ) // GREEK KAPPA SYMBOL
        return 922;
    if( c == 1009 ) // GREEK RHO SYMBOL
        return 929;
    if( c == 1010 ) // GREEK LUNATE SIGMA SYMBOL
        return 1017;
    if( c == 1013 ) // GREEK LUNATE EPSILON SYMBOL
        return 917;
    if( c == 1016 ) // GREEK SMALL LETTER SHO
        return 1015;
    if( c == 1019 ) // GREEK SMALL LETTER SAN
        return 1018;
    if( c >= 1072 && c <= 1103 ) // CYRILLIC SMALL LETTER A -> CYRILLIC SMALL LETTER YA
        return 1040+(c-1072);
    if( c >= 1104 && c <= 1119 ) // CYRILLIC SMALL LETTER IE WITH GRAVE -> CYRILLIC SMALL LETTER DZHE
        return 1024+(c-1104);
    if( c == 1121 ) // CYRILLIC SMALL LETTER OMEGA
        return 1120;
    if( c == 1123 ) // CYRILLIC SMALL LETTER YAT
        return 1122;
    if( c == 1125 ) // CYRILLIC SMALL LETTER IOTIFIED E
        return 1124;
    if( c == 1127 ) // CYRILLIC SMALL LETTER LITTLE YUS
        return 1126;
    if( c == 1129 ) // CYRILLIC SMALL LETTER IOTIFIED LITTLE YUS
        return 1128;
    if( c == 1131 ) // CYRILLIC SMALL LETTER BIG YUS
        return 1130;
    if( c == 1133 ) // CYRILLIC SMALL LETTER IOTIFIED BIG YUS
        return 1132;
    if( c == 1135 ) // CYRILLIC SMALL LETTER KSI
        return 1134;
    if( c == 1137 ) // CYRILLIC SMALL LETTER PSI
        return 1136;
    if( c == 1139 ) // CYRILLIC SMALL LETTER FITA
        return 1138;
    if( c == 1141 ) // CYRILLIC SMALL LETTER IZHITSA
        return 1140;
    if( c == 1143 ) // CYRILLIC SMALL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT
        return 1142;
    if( c == 1145 ) // CYRILLIC SMALL LETTER UK
        return 1144;
    if( c == 1147 ) // CYRILLIC SMALL LETTER ROUND OMEGA
        return 1146;
    if( c == 1149 ) // CYRILLIC SMALL LETTER OMEGA WITH TITLO
        return 1148;
    if( c == 1151 ) // CYRILLIC SMALL LETTER OT
        return 1150;
    if( c == 1153 ) // CYRILLIC SMALL LETTER KOPPA
        return 1152;
    if( c == 1163 ) // CYRILLIC SMALL LETTER SHORT I WITH TAIL
        return 1162;
    if( c == 1165 ) // CYRILLIC SMALL LETTER SEMISOFT SIGN
        return 1164;
    if( c == 1167 ) // CYRILLIC SMALL LETTER ER WITH TICK
        return 1166;
    if( c == 1169 ) // CYRILLIC SMALL LETTER GHE WITH UPTURN
        return 1168;
    if( c == 1171 ) // CYRILLIC SMALL LETTER GHE WITH STROKE
        return 1170;
    if( c == 1173 ) // CYRILLIC SMALL LETTER GHE WITH MIDDLE HOOK
        return 1172;
    if( c == 1175 ) // CYRILLIC SMALL LETTER ZHE WITH DESCENDER
        return 1174;
    if( c == 1177 ) // CYRILLIC SMALL LETTER ZE WITH DESCENDER
        return 1176;
    if( c == 1179 ) // CYRILLIC SMALL LETTER KA WITH DESCENDER
        return 1178;
    if( c == 1181 ) // CYRILLIC SMALL LETTER KA WITH VERTICAL STROKE
        return 1180;
    if( c == 1183 ) // CYRILLIC SMALL LETTER KA WITH STROKE
        return 1182;
    if( c == 1185 ) // CYRILLIC SMALL LETTER BASHKIR KA
        return 1184;
    if( c == 1187 ) // CYRILLIC SMALL LETTER EN WITH DESCENDER
        return 1186;
    if( c == 1189 ) // CYRILLIC SMALL LIGATURE EN GHE
        return 1188;
    if( c == 1191 ) // CYRILLIC SMALL LETTER PE WITH MIDDLE HOOK
        return 1190;
    if( c == 1193 ) // CYRILLIC SMALL LETTER ABKHASIAN HA
        return 1192;
    if( c == 1195 ) // CYRILLIC SMALL LETTER ES WITH DESCENDER
        return 1194;
    if( c == 1197 ) // CYRILLIC SMALL LETTER TE WITH DESCENDER
        return 1196;
    if( c == 1199 ) // CYRILLIC SMALL LETTER STRAIGHT U
        return 1198;
    if( c == 1201 ) // CYRILLIC SMALL LETTER STRAIGHT U WITH STROKE
        return 1200;
    if( c == 1203 ) // CYRILLIC SMALL LETTER HA WITH DESCENDER
        return 1202;
    if( c == 1205 ) // CYRILLIC SMALL LIGATURE TE TSE
        return 1204;
    if( c == 1207 ) // CYRILLIC SMALL LETTER CHE WITH DESCENDER
        return 1206;
    if( c == 1209 ) // CYRILLIC SMALL LETTER CHE WITH VERTICAL STROKE
        return 1208;
    if( c == 1211 ) // CYRILLIC SMALL LETTER SHHA
        return 1210;
    if( c == 1213 ) // CYRILLIC SMALL LETTER ABKHASIAN CHE
        return 1212;
    if( c == 1215 ) // CYRILLIC SMALL LETTER ABKHASIAN CHE WITH DESCENDER
        return 1214;
    if( c == 1218 ) // CYRILLIC SMALL LETTER ZHE WITH BREVE
        return 1217;
    if( c == 1220 ) // CYRILLIC SMALL LETTER KA WITH HOOK
        return 1219;
    if( c == 1222 ) // CYRILLIC SMALL LETTER EL WITH TAIL
        return 1221;
    if( c == 1224 ) // CYRILLIC SMALL LETTER EN WITH HOOK
        return 1223;
    if( c == 1226 ) // CYRILLIC SMALL LETTER EN WITH TAIL
        return 1225;
    if( c == 1228 ) // CYRILLIC SMALL LETTER KHAKASSIAN CHE
        return 1227;
    if( c == 1230 ) // CYRILLIC SMALL LETTER EM WITH TAIL
        return 1229;
    if( c == 1231 ) // CYRILLIC SMALL LETTER PALOCHKA
        return 1216;
    if( c == 1233 ) // CYRILLIC SMALL LETTER A WITH BREVE
        return 1232;
    if( c == 1235 ) // CYRILLIC SMALL LETTER A WITH DIAERESIS
        return 1234;
    if( c == 1237 ) // CYRILLIC SMALL LIGATURE A IE
        return 1236;
    if( c == 1239 ) // CYRILLIC SMALL LETTER IE WITH BREVE
        return 1238;
    if( c == 1241 ) // CYRILLIC SMALL LETTER SCHWA
        return 1240;
    if( c == 1243 ) // CYRILLIC SMALL LETTER SCHWA WITH DIAERESIS
        return 1242;
    if( c == 1245 ) // CYRILLIC SMALL LETTER ZHE WITH DIAERESIS
        return 1244;
    if( c == 1247 ) // CYRILLIC SMALL LETTER ZE WITH DIAERESIS
        return 1246;
    if( c == 1249 ) // CYRILLIC SMALL LETTER ABKHASIAN DZE
        return 1248;
    if( c == 1251 ) // CYRILLIC SMALL LETTER I WITH MACRON
        return 1250;
    if( c == 1253 ) // CYRILLIC SMALL LETTER I WITH DIAERESIS
        return 1252;
    if( c == 1255 ) // CYRILLIC SMALL LETTER O WITH DIAERESIS
        return 1254;
    if( c == 1257 ) // CYRILLIC SMALL LETTER BARRED O
        return 1256;
    if( c == 1259 ) // CYRILLIC SMALL LETTER BARRED O WITH DIAERESIS
        return 1258;
    if( c == 1261 ) // CYRILLIC SMALL LETTER E WITH DIAERESIS
        return 1260;
    if( c == 1263 ) // CYRILLIC SMALL LETTER U WITH MACRON
        return 1262;
    if( c == 1265 ) // CYRILLIC SMALL LETTER U WITH DIAERESIS
        return 1264;
    if( c == 1267 ) // CYRILLIC SMALL LETTER U WITH DOUBLE ACUTE
        return 1266;
    if( c == 1269 ) // CYRILLIC SMALL LETTER CHE WITH DIAERESIS
        return 1268;
    if( c == 1271 ) // CYRILLIC SMALL LETTER GHE WITH DESCENDER
        return 1270;
    if( c == 1273 ) // CYRILLIC SMALL LETTER YERU WITH DIAERESIS
        return 1272;
    if( c == 1275 ) // CYRILLIC SMALL LETTER GHE WITH STROKE AND HOOK
        return 1274;
    if( c == 1277 ) // CYRILLIC SMALL LETTER HA WITH HOOK
        return 1276;
    if( c == 1279 ) // CYRILLIC SMALL LETTER HA WITH STROKE
        return 1278;
    if( c == 1281 ) // CYRILLIC SMALL LETTER KOMI DE
        return 1280;
    if( c == 1283 ) // CYRILLIC SMALL LETTER KOMI DJE
        return 1282;
    if( c == 1285 ) // CYRILLIC SMALL LETTER KOMI ZJE
        return 1284;
    if( c == 1287 ) // CYRILLIC SMALL LETTER KOMI DZJE
        return 1286;
    if( c == 1289 ) // CYRILLIC SMALL LETTER KOMI LJE
        return 1288;
    if( c == 1291 ) // CYRILLIC SMALL LETTER KOMI NJE
        return 1290;
    if( c == 1293 ) // CYRILLIC SMALL LETTER KOMI SJE
        return 1292;
    if( c == 1295 ) // CYRILLIC SMALL LETTER KOMI TJE
        return 1294;
    if( c == 1297 ) // CYRILLIC SMALL LETTER REVERSED ZE
        return 1296;
    if( c == 1299 ) // CYRILLIC SMALL LETTER EL WITH HOOK
        return 1298;
    if( c == 1301 ) // CYRILLIC SMALL LETTER LHA
        return 1300;
    if( c == 1303 ) // CYRILLIC SMALL LETTER RHA
        return 1302;
    if( c == 1305 ) // CYRILLIC SMALL LETTER YAE
        return 1304;
    if( c == 1307 ) // CYRILLIC SMALL LETTER QA
        return 1306;
    if( c == 1309 ) // CYRILLIC SMALL LETTER WE
        return 1308;
    if( c == 1311 ) // CYRILLIC SMALL LETTER ALEUT KA
        return 1310;
    if( c == 1313 ) // CYRILLIC SMALL LETTER EL WITH MIDDLE HOOK
        return 1312;
    if( c == 1315 ) // CYRILLIC SMALL LETTER EN WITH MIDDLE HOOK
        return 1314;
    if( c >= 1377 && c <= 1414 ) // ARMENIAN SMALL LETTER AYB -> ARMENIAN SMALL LETTER FEH
        return 1329+(c-1377);
    if( c == 7545 ) // LATIN SMALL LETTER INSULAR G
        return 42877;
    if( c == 7549 ) // LATIN SMALL LETTER P WITH STROKE
        return 11363;
    if( c == 7681 ) // LATIN SMALL LETTER A WITH RING BELOW
        return 7680;
    if( c == 7683 ) // LATIN SMALL LETTER B WITH DOT ABOVE
        return 7682;
    if( c == 7685 ) // LATIN SMALL LETTER B WITH DOT BELOW
        return 7684;
    if( c == 7687 ) // LATIN SMALL LETTER B WITH LINE BELOW
        return 7686;
    if( c == 7689 ) // LATIN SMALL LETTER C WITH CEDILLA AND ACUTE
        return 7688;
    if( c == 7691 ) // LATIN SMALL LETTER D WITH DOT ABOVE
        return 7690;
    if( c == 7693 ) // LATIN SMALL LETTER D WITH DOT BELOW
        return 7692;
    if( c == 7695 ) // LATIN SMALL LETTER D WITH LINE BELOW
        return 7694;
    if( c == 7697 ) // LATIN SMALL LETTER D WITH CEDILLA
        return 7696;
    if( c == 7699 ) // LATIN SMALL LETTER D WITH CIRCUMFLEX BELOW
        return 7698;
    if( c == 7701 ) // LATIN SMALL LETTER E WITH MACRON AND GRAVE
        return 7700;
    if( c == 7703 ) // LATIN SMALL LETTER E WITH MACRON AND ACUTE
        return 7702;
    if( c == 7705 ) // LATIN SMALL LETTER E WITH CIRCUMFLEX BELOW
        return 7704;
    if( c == 7707 ) // LATIN SMALL LETTER E WITH TILDE BELOW
        return 7706;
    if( c == 7709 ) // LATIN SMALL LETTER E WITH CEDILLA AND BREVE
        return 7708;
    if( c == 7711 ) // LATIN SMALL LETTER F WITH DOT ABOVE
        return 7710;
    if( c == 7713 ) // LATIN SMALL LETTER G WITH MACRON
        return 7712;
    if( c == 7715 ) // LATIN SMALL LETTER H WITH DOT ABOVE
        return 7714;
    if( c == 7717 ) // LATIN SMALL LETTER H WITH DOT BELOW
        return 7716;
    if( c == 7719 ) // LATIN SMALL LETTER H WITH DIAERESIS
        return 7718;
    if( c == 7721 ) // LATIN SMALL LETTER H WITH CEDILLA
        return 7720;
    if( c == 7723 ) // LATIN SMALL LETTER H WITH BREVE BELOW
        return 7722;
    if( c == 7725 ) // LATIN SMALL LETTER I WITH TILDE BELOW
        return 7724;
    if( c == 7727 ) // LATIN SMALL LETTER I WITH DIAERESIS AND ACUTE
        return 7726;
    if( c == 7729 ) // LATIN SMALL LETTER K WITH ACUTE
        return 7728;
    if( c == 7731 ) // LATIN SMALL LETTER K WITH DOT BELOW
        return 7730;
    if( c == 7733 ) // LATIN SMALL LETTER K WITH LINE BELOW
        return 7732;
    if( c == 7735 ) // LATIN SMALL LETTER L WITH DOT BELOW
        return 7734;
    if( c == 7737 ) // LATIN SMALL LETTER L WITH DOT BELOW AND MACRON
        return 7736;
    if( c == 7739 ) // LATIN SMALL LETTER L WITH LINE BELOW
        return 7738;
    if( c == 7741 ) // LATIN SMALL LETTER L WITH CIRCUMFLEX BELOW
        return 7740;
    if( c == 7743 ) // LATIN SMALL LETTER M WITH ACUTE
        return 7742;
    if( c == 7745 ) // LATIN SMALL LETTER M WITH DOT ABOVE
        return 7744;
    if( c == 7747 ) // LATIN SMALL LETTER M WITH DOT BELOW
        return 7746;
    if( c == 7749 ) // LATIN SMALL LETTER N WITH DOT ABOVE
        return 7748;
    if( c == 7751 ) // LATIN SMALL LETTER N WITH DOT BELOW
        return 7750;
    if( c == 7753 ) // LATIN SMALL LETTER N WITH LINE BELOW
        return 7752;
    if( c == 7755 ) // LATIN SMALL LETTER N WITH CIRCUMFLEX BELOW
        return 7754;
    if( c == 7757 ) // LATIN SMALL LETTER O WITH TILDE AND ACUTE
        return 7756;
    if( c == 7759 ) // LATIN SMALL LETTER O WITH TILDE AND DIAERESIS
        return 7758;
    if( c == 7761 ) // LATIN SMALL LETTER O WITH MACRON AND GRAVE
        return 7760;
    if( c == 7763 ) // LATIN SMALL LETTER O WITH MACRON AND ACUTE
        return 7762;
    if( c == 7765 ) // LATIN SMALL LETTER P WITH ACUTE
        return 7764;
    if( c == 7767 ) // LATIN SMALL LETTER P WITH DOT ABOVE
        return 7766;
    if( c == 7769 ) // LATIN SMALL LETTER R WITH DOT ABOVE
        return 7768;
    if( c == 7771 ) // LATIN SMALL LETTER R WITH DOT BELOW
        return 7770;
    if( c == 7773 ) // LATIN SMALL LETTER R WITH DOT BELOW AND MACRON
        return 7772;
    if( c == 7775 ) // LATIN SMALL LETTER R WITH LINE BELOW
        return 7774;
    if( c == 7777 ) // LATIN SMALL LETTER S WITH DOT ABOVE
        return 7776;
    if( c == 7779 ) // LATIN SMALL LETTER S WITH DOT BELOW
        return 7778;
    if( c == 7781 ) // LATIN SMALL LETTER S WITH ACUTE AND DOT ABOVE
        return 7780;
    if( c == 7783 ) // LATIN SMALL LETTER S WITH CARON AND DOT ABOVE
        return 7782;
    if( c == 7785 ) // LATIN SMALL LETTER S WITH DOT BELOW AND DOT ABOVE
        return 7784;
    if( c == 7787 ) // LATIN SMALL LETTER T WITH DOT ABOVE
        return 7786;
    if( c == 7789 ) // LATIN SMALL LETTER T WITH DOT BELOW
        return 7788;
    if( c == 7791 ) // LATIN SMALL LETTER T WITH LINE BELOW
        return 7790;
    if( c == 7793 ) // LATIN SMALL LETTER T WITH CIRCUMFLEX BELOW
        return 7792;
    if( c == 7795 ) // LATIN SMALL LETTER U WITH DIAERESIS BELOW
        return 7794;
    if( c == 7797 ) // LATIN SMALL LETTER U WITH TILDE BELOW
        return 7796;
    if( c == 7799 ) // LATIN SMALL LETTER U WITH CIRCUMFLEX BELOW
        return 7798;
    if( c == 7801 ) // LATIN SMALL LETTER U WITH TILDE AND ACUTE
        return 7800;
    if( c == 7803 ) // LATIN SMALL LETTER U WITH MACRON AND DIAERESIS
        return 7802;
    if( c == 7805 ) // LATIN SMALL LETTER V WITH TILDE
        return 7804;
    if( c == 7807 ) // LATIN SMALL LETTER V WITH DOT BELOW
        return 7806;
    if( c == 7809 ) // LATIN SMALL LETTER W WITH GRAVE
        return 7808;
    if( c == 7811 ) // LATIN SMALL LETTER W WITH ACUTE
        return 7810;
    if( c == 7813 ) // LATIN SMALL LETTER W WITH DIAERESIS
        return 7812;
    if( c == 7815 ) // LATIN SMALL LETTER W WITH DOT ABOVE
        return 7814;
    if( c == 7817 ) // LATIN SMALL LETTER W WITH DOT BELOW
        return 7816;
    if( c == 7819 ) // LATIN SMALL LETTER X WITH DOT ABOVE
        return 7818;
    if( c == 7821 ) // LATIN SMALL LETTER X WITH DIAERESIS
        return 7820;
    if( c == 7823 ) // LATIN SMALL LETTER Y WITH DOT ABOVE
        return 7822;
    if( c == 7825 ) // LATIN SMALL LETTER Z WITH CIRCUMFLEX
        return 7824;
    if( c == 7827 ) // LATIN SMALL LETTER Z WITH DOT BELOW
        return 7826;
    if( c == 7829 ) // LATIN SMALL LETTER Z WITH LINE BELOW
        return 7828;
    if( c == 7835 ) // LATIN SMALL LETTER LONG S WITH DOT ABOVE
        return 7776;
    if( c == 7841 ) // LATIN SMALL LETTER A WITH DOT BELOW
        return 7840;
    if( c == 7843 ) // LATIN SMALL LETTER A WITH HOOK ABOVE
        return 7842;
    if( c == 7845 ) // LATIN SMALL LETTER A WITH CIRCUMFLEX AND ACUTE
        return 7844;
    if( c == 7847 ) // LATIN SMALL LETTER A WITH CIRCUMFLEX AND GRAVE
        return 7846;
    if( c == 7849 ) // LATIN SMALL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE
        return 7848;
    if( c == 7851 ) // LATIN SMALL LETTER A WITH CIRCUMFLEX AND TILDE
        return 7850;
    if( c == 7853 ) // LATIN SMALL LETTER A WITH CIRCUMFLEX AND DOT BELOW
        return 7852;
    if( c == 7855 ) // LATIN SMALL LETTER A WITH BREVE AND ACUTE
        return 7854;
    if( c == 7857 ) // LATIN SMALL LETTER A WITH BREVE AND GRAVE
        return 7856;
    if( c == 7859 ) // LATIN SMALL LETTER A WITH BREVE AND HOOK ABOVE
        return 7858;
    if( c == 7861 ) // LATIN SMALL LETTER A WITH BREVE AND TILDE
        return 7860;
    if( c == 7863 ) // LATIN SMALL LETTER A WITH BREVE AND DOT BELOW
        return 7862;
    if( c == 7865 ) // LATIN SMALL LETTER E WITH DOT BELOW
        return 7864;
    if( c == 7867 ) // LATIN SMALL LETTER E WITH HOOK ABOVE
        return 7866;
    if( c == 7869 ) // LATIN SMALL LETTER E WITH TILDE
        return 7868;
    if( c == 7871 ) // LATIN SMALL LETTER E WITH CIRCUMFLEX AND ACUTE
        return 7870;
    if( c == 7873 ) // LATIN SMALL LETTER E WITH CIRCUMFLEX AND GRAVE
        return 7872;
    if( c == 7875 ) // LATIN SMALL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE
        return 7874;
    if( c == 7877 ) // LATIN SMALL LETTER E WITH CIRCUMFLEX AND TILDE
        return 7876;
    if( c == 7879 ) // LATIN SMALL LETTER E WITH CIRCUMFLEX AND DOT BELOW
        return 7878;
    if( c == 7881 ) // LATIN SMALL LETTER I WITH HOOK ABOVE
        return 7880;
    if( c == 7883 ) // LATIN SMALL LETTER I WITH DOT BELOW
        return 7882;
    if( c == 7885 ) // LATIN SMALL LETTER O WITH DOT BELOW
        return 7884;
    if( c == 7887 ) // LATIN SMALL LETTER O WITH HOOK ABOVE
        return 7886;
    if( c == 7889 ) // LATIN SMALL LETTER O WITH CIRCUMFLEX AND ACUTE
        return 7888;
    if( c == 7891 ) // LATIN SMALL LETTER O WITH CIRCUMFLEX AND GRAVE
        return 7890;
    if( c == 7893 ) // LATIN SMALL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE
        return 7892;
    if( c == 7895 ) // LATIN SMALL LETTER O WITH CIRCUMFLEX AND TILDE
        return 7894;
    if( c == 7897 ) // LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW
        return 7896;
    if( c == 7899 ) // LATIN SMALL LETTER O WITH HORN AND ACUTE
        return 7898;
    if( c == 7901 ) // LATIN SMALL LETTER O WITH HORN AND GRAVE
        return 7900;
    if( c == 7903 ) // LATIN SMALL LETTER O WITH HORN AND HOOK ABOVE
        return 7902;
    if( c == 7905 ) // LATIN SMALL LETTER O WITH HORN AND TILDE
        return 7904;
    if( c == 7907 ) // LATIN SMALL LETTER O WITH HORN AND DOT BELOW
        return 7906;
    if( c == 7909 ) // LATIN SMALL LETTER U WITH DOT BELOW
        return 7908;
    if( c == 7911 ) // LATIN SMALL LETTER U WITH HOOK ABOVE
        return 7910;
    if( c == 7913 ) // LATIN SMALL LETTER U WITH HORN AND ACUTE
        return 7912;
    if( c == 7915 ) // LATIN SMALL LETTER U WITH HORN AND GRAVE
        return 7914;
    if( c == 7917 ) // LATIN SMALL LETTER U WITH HORN AND HOOK ABOVE
        return 7916;
    if( c == 7919 ) // LATIN SMALL LETTER U WITH HORN AND TILDE
        return 7918;
    if( c == 7921 ) // LATIN SMALL LETTER U WITH HORN AND DOT BELOW
        return 7920;
    if( c == 7923 ) // LATIN SMALL LETTER Y WITH GRAVE
        return 7922;
    if( c == 7925 ) // LATIN SMALL LETTER Y WITH DOT BELOW
        return 7924;
    if( c == 7927 ) // LATIN SMALL LETTER Y WITH HOOK ABOVE
        return 7926;
    if( c == 7929 ) // LATIN SMALL LETTER Y WITH TILDE
        return 7928;
    if( c == 7931 ) // LATIN SMALL LETTER MIDDLE-WELSH LL
        return 7930;
    if( c == 7933 ) // LATIN SMALL LETTER MIDDLE-WELSH V
        return 7932;
    if( c == 7935 ) // LATIN SMALL LETTER Y WITH LOOP
        return 7934;
    if( c >= 7936 && c <= 7943 ) // GREEK SMALL LETTER ALPHA WITH PSILI -> GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI
        return 7944+(c-7936);
    if( c >= 7952 && c <= 7957 ) // GREEK SMALL LETTER EPSILON WITH PSILI -> GREEK SMALL LETTER EPSILON WITH DASIA AND OXIA
        return 7960+(c-7952);
    if( c >= 7968 && c <= 7975 ) // GREEK SMALL LETTER ETA WITH PSILI -> GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI
        return 7976+(c-7968);
    if( c >= 7984 && c <= 7991 ) // GREEK SMALL LETTER IOTA WITH PSILI -> GREEK SMALL LETTER IOTA WITH DASIA AND PERISPOMENI
        return 7992+(c-7984);
    if( c >= 8000 && c <= 8005 ) // GREEK SMALL LETTER OMICRON WITH PSILI -> GREEK SMALL LETTER OMICRON WITH DASIA AND OXIA
        return 8008+(c-8000);
    if( c == 8017 ) // GREEK SMALL LETTER UPSILON WITH DASIA
        return 8025;
    if( c == 8019 ) // GREEK SMALL LETTER UPSILON WITH DASIA AND VARIA
        return 8027;
    if( c == 8021 ) // GREEK SMALL LETTER UPSILON WITH DASIA AND OXIA
        return 8029;
    if( c == 8023 ) // GREEK SMALL LETTER UPSILON WITH DASIA AND PERISPOMENI
        return 8031;
    if( c >= 8032 && c <= 8039 ) // GREEK SMALL LETTER OMEGA WITH PSILI -> GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI
        return 8040+(c-8032);
    if( c >= 8048 && c <= 8049 ) // GREEK SMALL LETTER ALPHA WITH VARIA -> GREEK SMALL LETTER ALPHA WITH OXIA
        return 8122+(c-8048);
    if( c >= 8050 && c <= 8053 ) // GREEK SMALL LETTER EPSILON WITH VARIA -> GREEK SMALL LETTER ETA WITH OXIA
        return 8136+(c-8050);
    if( c >= 8054 && c <= 8055 ) // GREEK SMALL LETTER IOTA WITH VARIA -> GREEK SMALL LETTER IOTA WITH OXIA
        return 8154+(c-8054);
    if( c >= 8056 && c <= 8057 ) // GREEK SMALL LETTER OMICRON WITH VARIA -> GREEK SMALL LETTER OMICRON WITH OXIA
        return 8184+(c-8056);
    if( c >= 8058 && c <= 8059 ) // GREEK SMALL LETTER UPSILON WITH VARIA -> GREEK SMALL LETTER UPSILON WITH OXIA
        return 8170+(c-8058);
    if( c >= 8060 && c <= 8061 ) // GREEK SMALL LETTER OMEGA WITH VARIA -> GREEK SMALL LETTER OMEGA WITH OXIA
        return 8186+(c-8060);
    if( c >= 8064 && c <= 8071 ) // GREEK SMALL LETTER ALPHA WITH PSILI AND YPOGEGRAMMENI -> GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI
        return 8072+(c-8064);
    if( c >= 8080 && c <= 8087 ) // GREEK SMALL LETTER ETA WITH PSILI AND YPOGEGRAMMENI -> GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI
        return 8088+(c-8080);
    if( c >= 8096 && c <= 8103 ) // GREEK SMALL LETTER OMEGA WITH PSILI AND YPOGEGRAMMENI -> GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI
        return 8104+(c-8096);
    if( c >= 8112 && c <= 8113 ) // GREEK SMALL LETTER ALPHA WITH VRACHY -> GREEK SMALL LETTER ALPHA WITH MACRON
        return 8120+(c-8112);
    if( c == 8115 ) // GREEK SMALL LETTER ALPHA WITH YPOGEGRAMMENI
        return 8124;
    if( c == 8126 ) // GREEK PROSGEGRAMMENI
        return 921;
    if( c == 8131 ) // GREEK SMALL LETTER ETA WITH YPOGEGRAMMENI
        return 8140;
    if( c >= 8144 && c <= 8145 ) // GREEK SMALL LETTER IOTA WITH VRACHY -> GREEK SMALL LETTER IOTA WITH MACRON
        return 8152+(c-8144);
    if( c >= 8160 && c <= 8161 ) // GREEK SMALL LETTER UPSILON WITH VRACHY -> GREEK SMALL LETTER UPSILON WITH MACRON
        return 8168+(c-8160);
    if( c == 8165 ) // GREEK SMALL LETTER RHO WITH DASIA
        return 8172;
    if( c == 8179 ) // GREEK SMALL LETTER OMEGA WITH YPOGEGRAMMENI
        return 8188;
    if( c == 8526 ) // TURNED SMALL F
        return 8498;
    if( c == 8580 ) // LATIN SMALL LETTER REVERSED C
        return 8579;
    if( c >= 11312 && c <= 11358 ) // GLAGOLITIC SMALL LETTER AZU -> GLAGOLITIC SMALL LETTER LATINATE MYSLITE
        return 11264+(c-11312);
    if( c == 11361 ) // LATIN SMALL LETTER L WITH DOUBLE BAR
        return 11360;
    if( c == 11365 ) // LATIN SMALL LETTER A WITH STROKE
        return 570;
    if( c == 11366 ) // LATIN SMALL LETTER T WITH DIAGONAL STROKE
        return 574;
    if( c == 11368 ) // LATIN SMALL LETTER H WITH DESCENDER
        return 11367;
    if( c == 11370 ) // LATIN SMALL LETTER K WITH DESCENDER
        return 11369;
    if( c == 11372 ) // LATIN SMALL LETTER Z WITH DESCENDER
        return 11371;
    if( c == 11379 ) // LATIN SMALL LETTER W WITH HOOK
        return 11378;
    if( c == 11382 ) // LATIN SMALL LETTER HALF H
        return 11381;
    if( c == 11393 ) // COPTIC SMALL LETTER ALFA
        return 11392;
    if( c == 11395 ) // COPTIC SMALL LETTER VIDA
        return 11394;
    if( c == 11397 ) // COPTIC SMALL LETTER GAMMA
        return 11396;
    if( c == 11399 ) // COPTIC SMALL LETTER DALDA
        return 11398;
    if( c == 11401 ) // COPTIC SMALL LETTER EIE
        return 11400;
    if( c == 11403 ) // COPTIC SMALL LETTER SOU
        return 11402;
    if( c == 11405 ) // COPTIC SMALL LETTER ZATA
        return 11404;
    if( c == 11407 ) // COPTIC SMALL LETTER HATE
        return 11406;
    if( c == 11409 ) // COPTIC SMALL LETTER THETHE
        return 11408;
    if( c == 11411 ) // COPTIC SMALL LETTER IAUDA
        return 11410;
    if( c == 11413 ) // COPTIC SMALL LETTER KAPA
        return 11412;
    if( c == 11415 ) // COPTIC SMALL LETTER LAULA
        return 11414;
    if( c == 11417 ) // COPTIC SMALL LETTER MI
        return 11416;
    if( c == 11419 ) // COPTIC SMALL LETTER NI
        return 11418;
    if( c == 11421 ) // COPTIC SMALL LETTER KSI
        return 11420;
    if( c == 11423 ) // COPTIC SMALL LETTER O
        return 11422;
    if( c == 11425 ) // COPTIC SMALL LETTER PI
        return 11424;
    if( c == 11427 ) // COPTIC SMALL LETTER RO
        return 11426;
    if( c == 11429 ) // COPTIC SMALL LETTER SIMA
        return 11428;
    if( c == 11431 ) // COPTIC SMALL LETTER TAU
        return 11430;
    if( c == 11433 ) // COPTIC SMALL LETTER UA
        return 11432;
    if( c == 11435 ) // COPTIC SMALL LETTER FI
        return 11434;
    if( c == 11437 ) // COPTIC SMALL LETTER KHI
        return 11436;
    if( c == 11439 ) // COPTIC SMALL LETTER PSI
        return 11438;
    if( c == 11441 ) // COPTIC SMALL LETTER OOU
        return 11440;
    if( c == 11443 ) // COPTIC SMALL LETTER DIALECT-P ALEF
        return 11442;
    if( c == 11445 ) // COPTIC SMALL LETTER OLD COPTIC AIN
        return 11444;
    if( c == 11447 ) // COPTIC SMALL LETTER CRYPTOGRAMMIC EIE
        return 11446;
    if( c == 11449 ) // COPTIC SMALL LETTER DIALECT-P KAPA
        return 11448;
    if( c == 11451 ) // COPTIC SMALL LETTER DIALECT-P NI
        return 11450;
    if( c == 11453 ) // COPTIC SMALL LETTER CRYPTOGRAMMIC NI
        return 11452;
    if( c == 11455 ) // COPTIC SMALL LETTER OLD COPTIC OOU
        return 11454;
    if( c == 11457 ) // COPTIC SMALL LETTER SAMPI
        return 11456;
    if( c == 11459 ) // COPTIC SMALL LETTER CROSSED SHEI
        return 11458;
    if( c == 11461 ) // COPTIC SMALL LETTER OLD COPTIC SHEI
        return 11460;
    if( c == 11463 ) // COPTIC SMALL LETTER OLD COPTIC ESH
        return 11462;
    if( c == 11465 ) // COPTIC SMALL LETTER AKHMIMIC KHEI
        return 11464;
    if( c == 11467 ) // COPTIC SMALL LETTER DIALECT-P HORI
        return 11466;
    if( c == 11469 ) // COPTIC SMALL LETTER OLD COPTIC HORI
        return 11468;
    if( c == 11471 ) // COPTIC SMALL LETTER OLD COPTIC HA
        return 11470;
    if( c == 11473 ) // COPTIC SMALL LETTER L-SHAPED HA
        return 11472;
    if( c == 11475 ) // COPTIC SMALL LETTER OLD COPTIC HEI
        return 11474;
    if( c == 11477 ) // COPTIC SMALL LETTER OLD COPTIC HAT
        return 11476;
    if( c == 11479 ) // COPTIC SMALL LETTER OLD COPTIC GANGIA
        return 11478;
    if( c == 11481 ) // COPTIC SMALL LETTER OLD COPTIC DJA
        return 11480;
    if( c == 11483 ) // COPTIC SMALL LETTER OLD COPTIC SHIMA
        return 11482;
    if( c == 11485 ) // COPTIC SMALL LETTER OLD NUBIAN SHIMA
        return 11484;
    if( c == 11487 ) // COPTIC SMALL LETTER OLD NUBIAN NGI
        return 11486;
    if( c == 11489 ) // COPTIC SMALL LETTER OLD NUBIAN NYI
        return 11488;
    if( c == 11491 ) // COPTIC SMALL LETTER OLD NUBIAN WAU
        return 11490;
    if( c >= 11520 && c <= 11557 ) // GEORGIAN SMALL LETTER AN -> GEORGIAN SMALL LETTER HOE
        return 4256+(c-11520);
    if( c == 42561 ) // CYRILLIC SMALL LETTER ZEMLYA
        return 42560;
    if( c == 42563 ) // CYRILLIC SMALL LETTER DZELO
        return 42562;
    if( c == 42565 ) // CYRILLIC SMALL LETTER REVERSED DZE
        return 42564;
    if( c == 42567 ) // CYRILLIC SMALL LETTER IOTA
        return 42566;
    if( c == 42569 ) // CYRILLIC SMALL LETTER DJERV
        return 42568;
    if( c == 42571 ) // CYRILLIC SMALL LETTER MONOGRAPH UK
        return 42570;
    if( c == 42573 ) // CYRILLIC SMALL LETTER BROAD OMEGA
        return 42572;
    if( c == 42575 ) // CYRILLIC SMALL LETTER NEUTRAL YER
        return 42574;
    if( c == 42577 ) // CYRILLIC SMALL LETTER YERU WITH BACK YER
        return 42576;
    if( c == 42579 ) // CYRILLIC SMALL LETTER IOTIFIED YAT
        return 42578;
    if( c == 42581 ) // CYRILLIC SMALL LETTER REVERSED YU
        return 42580;
    if( c == 42583 ) // CYRILLIC SMALL LETTER IOTIFIED A
        return 42582;
    if( c == 42585 ) // CYRILLIC SMALL LETTER CLOSED LITTLE YUS
        return 42584;
    if( c == 42587 ) // CYRILLIC SMALL LETTER BLENDED YUS
        return 42586;
    if( c == 42589 ) // CYRILLIC SMALL LETTER IOTIFIED CLOSED LITTLE YUS
        return 42588;
    if( c == 42591 ) // CYRILLIC SMALL LETTER YN
        return 42590;
    if( c == 42595 ) // CYRILLIC SMALL LETTER SOFT DE
        return 42594;
    if( c == 42597 ) // CYRILLIC SMALL LETTER SOFT EL
        return 42596;
    if( c == 42599 ) // CYRILLIC SMALL LETTER SOFT EM
        return 42598;
    if( c == 42601 ) // CYRILLIC SMALL LETTER MONOCULAR O
        return 42600;
    if( c == 42603 ) // CYRILLIC SMALL LETTER BINOCULAR O
        return 42602;
    if( c == 42605 ) // CYRILLIC SMALL LETTER DOUBLE MONOCULAR O
        return 42604;
    if( c == 42625 ) // CYRILLIC SMALL LETTER DWE
        return 42624;
    if( c == 42627 ) // CYRILLIC SMALL LETTER DZWE
        return 42626;
    if( c == 42629 ) // CYRILLIC SMALL LETTER ZHWE
        return 42628;
    if( c == 42631 ) // CYRILLIC SMALL LETTER CCHE
        return 42630;
    if( c == 42633 ) // CYRILLIC SMALL LETTER DZZE
        return 42632;
    if( c == 42635 ) // CYRILLIC SMALL LETTER TE WITH MIDDLE HOOK
        return 42634;
    if( c == 42637 ) // CYRILLIC SMALL LETTER TWE
        return 42636;
    if( c == 42639 ) // CYRILLIC SMALL LETTER TSWE
        return 42638;
    if( c == 42641 ) // CYRILLIC SMALL LETTER TSSE
        return 42640;
    if( c == 42643 ) // CYRILLIC SMALL LETTER TCHE
        return 42642;
    if( c == 42645 ) // CYRILLIC SMALL LETTER HWE
        return 42644;
    if( c == 42647 ) // CYRILLIC SMALL LETTER SHWE
        return 42646;
    if( c == 42787 ) // LATIN SMALL LETTER EGYPTOLOGICAL ALEF
        return 42786;
    if( c == 42789 ) // LATIN SMALL LETTER EGYPTOLOGICAL AIN
        return 42788;
    if( c == 42791 ) // LATIN SMALL LETTER HENG
        return 42790;
    if( c == 42793 ) // LATIN SMALL LETTER TZ
        return 42792;
    if( c == 42795 ) // LATIN SMALL LETTER TRESILLO
        return 42794;
    if( c == 42797 ) // LATIN SMALL LETTER CUATRILLO
        return 42796;
    if( c == 42799 ) // LATIN SMALL LETTER CUATRILLO WITH COMMA
        return 42798;
    if( c == 42803 ) // LATIN SMALL LETTER AA
        return 42802;
    if( c == 42805 ) // LATIN SMALL LETTER AO
        return 42804;
    if( c == 42807 ) // LATIN SMALL LETTER AU
        return 42806;
    if( c == 42809 ) // LATIN SMALL LETTER AV
        return 42808;
    if( c == 42811 ) // LATIN SMALL LETTER AV WITH HORIZONTAL BAR
        return 42810;
    if( c == 42813 ) // LATIN SMALL LETTER AY
        return 42812;
    if( c == 42815 ) // LATIN SMALL LETTER REVERSED C WITH DOT
        return 42814;
    if( c == 42817 ) // LATIN SMALL LETTER K WITH STROKE
        return 42816;
    if( c == 42819 ) // LATIN SMALL LETTER K WITH DIAGONAL STROKE
        return 42818;
    if( c == 42821 ) // LATIN SMALL LETTER K WITH STROKE AND DIAGONAL STROKE
        return 42820;
    if( c == 42823 ) // LATIN SMALL LETTER BROKEN L
        return 42822;
    if( c == 42825 ) // LATIN SMALL LETTER L WITH HIGH STROKE
        return 42824;
    if( c == 42827 ) // LATIN SMALL LETTER O WITH LONG STROKE OVERLAY
        return 42826;
    if( c == 42829 ) // LATIN SMALL LETTER O WITH LOOP
        return 42828;
    if( c == 42831 ) // LATIN SMALL LETTER OO
        return 42830;
    if( c == 42833 ) // LATIN SMALL LETTER P WITH STROKE THROUGH DESCENDER
        return 42832;
    if( c == 42835 ) // LATIN SMALL LETTER P WITH FLOURISH
        return 42834;
    if( c == 42837 ) // LATIN SMALL LETTER P WITH SQUIRREL TAIL
        return 42836;
    if( c == 42839 ) // LATIN SMALL LETTER Q WITH STROKE THROUGH DESCENDER
        return 42838;
    if( c == 42841 ) // LATIN SMALL LETTER Q WITH DIAGONAL STROKE
        return 42840;
    if( c == 42843 ) // LATIN SMALL LETTER R ROTUNDA
        return 42842;
    if( c == 42845 ) // LATIN SMALL LETTER RUM ROTUNDA
        return 42844;
    if( c == 42847 ) // LATIN SMALL LETTER V WITH DIAGONAL STROKE
        return 42846;
    if( c == 42849 ) // LATIN SMALL LETTER VY
        return 42848;
    if( c == 42851 ) // LATIN SMALL LETTER VISIGOTHIC Z
        return 42850;
    if( c == 42853 ) // LATIN SMALL LETTER THORN WITH STROKE
        return 42852;
    if( c == 42855 ) // LATIN SMALL LETTER THORN WITH STROKE THROUGH DESCENDER
        return 42854;
    if( c == 42857 ) // LATIN SMALL LETTER VEND
        return 42856;
    if( c == 42859 ) // LATIN SMALL LETTER ET
        return 42858;
    if( c == 42861 ) // LATIN SMALL LETTER IS
        return 42860;
    if( c == 42863 ) // LATIN SMALL LETTER CON
        return 42862;
    if( c == 42874 ) // LATIN SMALL LETTER INSULAR D
        return 42873;
    if( c == 42876 ) // LATIN SMALL LETTER INSULAR F
        return 42875;
    if( c == 42879 ) // LATIN SMALL LETTER TURNED INSULAR G
        return 42878;
    if( c == 42881 ) // LATIN SMALL LETTER TURNED L
        return 42880;
    if( c == 42883 ) // LATIN SMALL LETTER INSULAR R
        return 42882;
    if( c == 42885 ) // LATIN SMALL LETTER INSULAR S
        return 42884;
    if( c == 42887 ) // LATIN SMALL LETTER INSULAR T
        return 42886;
    if( c == 42892 ) // LATIN SMALL LETTER SALTILLO
        return 42891;
    if( c >= 65345 && c <= 65370 ) // FULLWIDTH LATIN SMALL LETTER A -> FULLWIDTH LATIN SMALL LETTER Z
        return 65313+(c-65345);
    if( c >= 66600 && c <= 66639 ) // DESERET SMALL LETTER LONG I -> DESERET SMALL LETTER EW
        return 66560+(c-66600);
    return c;
}

codepoint glr_tolower(codepoint c)
{
    if( c >= 65 && c <= 90 ) // LATIN CAPITAL LETTER A -> LATIN CAPITAL LETTER Z
        return 97+(c-65);
    if( c >= 192 && c <= 214 ) // LATIN CAPITAL LETTER A WITH GRAVE -> LATIN CAPITAL LETTER O WITH DIAERESIS
        return 224+(c-192);
    if( c >= 216 && c <= 222 ) // LATIN CAPITAL LETTER O WITH STROKE -> LATIN CAPITAL LETTER THORN
        return 248+(c-216);
    if( c == 256 ) // LATIN CAPITAL LETTER A WITH MACRON
        return 257;
    if( c == 258 ) // LATIN CAPITAL LETTER A WITH BREVE
        return 259;
    if( c == 260 ) // LATIN CAPITAL LETTER A WITH OGONEK
        return 261;
    if( c == 262 ) // LATIN CAPITAL LETTER C WITH ACUTE
        return 263;
    if( c == 264 ) // LATIN CAPITAL LETTER C WITH CIRCUMFLEX
        return 265;
    if( c == 266 ) // LATIN CAPITAL LETTER C WITH DOT ABOVE
        return 267;
    if( c == 268 ) // LATIN CAPITAL LETTER C WITH CARON
        return 269;
    if( c == 270 ) // LATIN CAPITAL LETTER D WITH CARON
        return 271;
    if( c == 272 ) // LATIN CAPITAL LETTER D WITH STROKE
        return 273;
    if( c == 274 ) // LATIN CAPITAL LETTER E WITH MACRON
        return 275;
    if( c == 276 ) // LATIN CAPITAL LETTER E WITH BREVE
        return 277;
    if( c == 278 ) // LATIN CAPITAL LETTER E WITH DOT ABOVE
        return 279;
    if( c == 280 ) // LATIN CAPITAL LETTER E WITH OGONEK
        return 281;
    if( c == 282 ) // LATIN CAPITAL LETTER E WITH CARON
        return 283;
    if( c == 284 ) // LATIN CAPITAL LETTER G WITH CIRCUMFLEX
        return 285;
    if( c == 286 ) // LATIN CAPITAL LETTER G WITH BREVE
        return 287;
    if( c == 288 ) // LATIN CAPITAL LETTER G WITH DOT ABOVE
        return 289;
    if( c == 290 ) // LATIN CAPITAL LETTER G WITH CEDILLA
        return 291;
    if( c == 292 ) // LATIN CAPITAL LETTER H WITH CIRCUMFLEX
        return 293;
    if( c == 294 ) // LATIN CAPITAL LETTER H WITH STROKE
        return 295;
    if( c == 296 ) // LATIN CAPITAL LETTER I WITH TILDE
        return 297;
    if( c == 298 ) // LATIN CAPITAL LETTER I WITH MACRON
        return 299;
    if( c == 300 ) // LATIN CAPITAL LETTER I WITH BREVE
        return 301;
    if( c == 302 ) // LATIN CAPITAL LETTER I WITH OGONEK
        return 303;
    if( c == 304 ) // LATIN CAPITAL LETTER I WITH DOT ABOVE
        return 105;
    if( c == 306 ) // LATIN CAPITAL LIGATURE IJ
        return 307;
    if( c == 308 ) // LATIN CAPITAL LETTER J WITH CIRCUMFLEX
        return 309;
    if( c == 310 ) // LATIN CAPITAL LETTER K WITH CEDILLA
        return 311;
    if( c == 313 ) // LATIN CAPITAL LETTER L WITH ACUTE
        return 314;
    if( c == 315 ) // LATIN CAPITAL LETTER L WITH CEDILLA
        return 316;
    if( c == 317 ) // LATIN CAPITAL LETTER L WITH CARON
        return 318;
    if( c == 319 ) // LATIN CAPITAL LETTER L WITH MIDDLE DOT
        return 320;
    if( c == 321 ) // LATIN CAPITAL LETTER L WITH STROKE
        return 322;
    if( c == 323 ) // LATIN CAPITAL LETTER N WITH ACUTE
        return 324;
    if( c == 325 ) // LATIN CAPITAL LETTER N WITH CEDILLA
        return 326;
    if( c == 327 ) // LATIN CAPITAL LETTER N WITH CARON
        return 328;
    if( c == 330 ) // LATIN CAPITAL LETTER ENG
        return 331;
    if( c == 332 ) // LATIN CAPITAL LETTER O WITH MACRON
        return 333;
    if( c == 334 ) // LATIN CAPITAL LETTER O WITH BREVE
        return 335;
    if( c == 336 ) // LATIN CAPITAL LETTER O WITH DOUBLE ACUTE
        return 337;
    if( c == 338 ) // LATIN CAPITAL LIGATURE OE
        return 339;
    if( c == 340 ) // LATIN CAPITAL LETTER R WITH ACUTE
        return 341;
    if( c == 342 ) // LATIN CAPITAL LETTER R WITH CEDILLA
        return 343;
    if( c == 344 ) // LATIN CAPITAL LETTER R WITH CARON
        return 345;
    if( c == 346 ) // LATIN CAPITAL LETTER S WITH ACUTE
        return 347;
    if( c == 348 ) // LATIN CAPITAL LETTER S WITH CIRCUMFLEX
        return 349;
    if( c == 350 ) // LATIN CAPITAL LETTER S WITH CEDILLA
        return 351;
    if( c == 352 ) // LATIN CAPITAL LETTER S WITH CARON
        return 353;
    if( c == 354 ) // LATIN CAPITAL LETTER T WITH CEDILLA
        return 355;
    if( c == 356 ) // LATIN CAPITAL LETTER T WITH CARON
        return 357;
    if( c == 358 ) // LATIN CAPITAL LETTER T WITH STROKE
        return 359;
    if( c == 360 ) // LATIN CAPITAL LETTER U WITH TILDE
        return 361;
    if( c == 362 ) // LATIN CAPITAL LETTER U WITH MACRON
        return 363;
    if( c == 364 ) // LATIN CAPITAL LETTER U WITH BREVE
        return 365;
    if( c == 366 ) // LATIN CAPITAL LETTER U WITH RING ABOVE
        return 367;
    if( c == 368 ) // LATIN CAPITAL LETTER U WITH DOUBLE ACUTE
        return 369;
    if( c == 370 ) // LATIN CAPITAL LETTER U WITH OGONEK
        return 371;
    if( c == 372 ) // LATIN CAPITAL LETTER W WITH CIRCUMFLEX
        return 373;
    if( c == 374 ) // LATIN CAPITAL LETTER Y WITH CIRCUMFLEX
        return 375;
    if( c == 376 ) // LATIN CAPITAL LETTER Y WITH DIAERESIS
        return 255;
    if( c == 377 ) // LATIN CAPITAL LETTER Z WITH ACUTE
        return 378;
    if( c == 379 ) // LATIN CAPITAL LETTER Z WITH DOT ABOVE
        return 380;
    if( c == 381 ) // LATIN CAPITAL LETTER Z WITH CARON
        return 382;
    if( c == 385 ) // LATIN CAPITAL LETTER B WITH HOOK
        return 595;
    if( c == 386 ) // LATIN CAPITAL LETTER B WITH TOPBAR
        return 387;
    if( c == 388 ) // LATIN CAPITAL LETTER TONE SIX
        return 389;
    if( c == 390 ) // LATIN CAPITAL LETTER OPEN O
        return 596;
    if( c == 391 ) // LATIN CAPITAL LETTER C WITH HOOK
        return 392;
    if( c >= 393 && c <= 394 ) // LATIN CAPITAL LETTER AFRICAN D -> LATIN CAPITAL LETTER D WITH HOOK
        return 598+(c-393);
    if( c == 395 ) // LATIN CAPITAL LETTER D WITH TOPBAR
        return 396;
    if( c == 398 ) // LATIN CAPITAL LETTER REVERSED E
        return 477;
    if( c == 399 ) // LATIN CAPITAL LETTER SCHWA
        return 601;
    if( c == 400 ) // LATIN CAPITAL LETTER OPEN E
        return 603;
    if( c == 401 ) // LATIN CAPITAL LETTER F WITH HOOK
        return 402;
    if( c == 403 ) // LATIN CAPITAL LETTER G WITH HOOK
        return 608;
    if( c == 404 ) // LATIN CAPITAL LETTER GAMMA
        return 611;
    if( c == 406 ) // LATIN CAPITAL LETTER IOTA
        return 617;
    if( c == 407 ) // LATIN CAPITAL LETTER I WITH STROKE
        return 616;
    if( c == 408 ) // LATIN CAPITAL LETTER K WITH HOOK
        return 409;
    if( c == 412 ) // LATIN CAPITAL LETTER TURNED M
        return 623;
    if( c == 413 ) // LATIN CAPITAL LETTER N WITH LEFT HOOK
        return 626;
    if( c == 415 ) // LATIN CAPITAL LETTER O WITH MIDDLE TILDE
        return 629;
    if( c == 416 ) // LATIN CAPITAL LETTER O WITH HORN
        return 417;
    if( c == 418 ) // LATIN CAPITAL LETTER OI
        return 419;
    if( c == 420 ) // LATIN CAPITAL LETTER P WITH HOOK
        return 421;
    if( c == 422 ) // LATIN LETTER YR
        return 640;
    if( c == 423 ) // LATIN CAPITAL LETTER TONE TWO
        return 424;
    if( c == 425 ) // LATIN CAPITAL LETTER ESH
        return 643;
    if( c == 428 ) // LATIN CAPITAL LETTER T WITH HOOK
        return 429;
    if( c == 430 ) // LATIN CAPITAL LETTER T WITH RETROFLEX HOOK
        return 648;
    if( c == 431 ) // LATIN CAPITAL LETTER U WITH HORN
        return 432;
    if( c >= 433 && c <= 434 ) // LATIN CAPITAL LETTER UPSILON -> LATIN CAPITAL LETTER V WITH HOOK
        return 650+(c-433);
    if( c == 435 ) // LATIN CAPITAL LETTER Y WITH HOOK
        return 436;
    if( c == 437 ) // LATIN CAPITAL LETTER Z WITH STROKE
        return 438;
    if( c == 439 ) // LATIN CAPITAL LETTER EZH
        return 658;
    if( c == 440 ) // LATIN CAPITAL LETTER EZH REVERSED
        return 441;
    if( c == 444 ) // LATIN CAPITAL LETTER TONE FIVE
        return 445;
    if( c == 452 ) // LATIN CAPITAL LETTER DZ WITH CARON
        return 454;
    if( c == 455 ) // LATIN CAPITAL LETTER LJ
        return 457;
    if( c == 458 ) // LATIN CAPITAL LETTER NJ
        return 460;
    if( c == 461 ) // LATIN CAPITAL LETTER A WITH CARON
        return 462;
    if( c == 463 ) // LATIN CAPITAL LETTER I WITH CARON
        return 464;
    if( c == 465 ) // LATIN CAPITAL LETTER O WITH CARON
        return 466;
    if( c == 467 ) // LATIN CAPITAL LETTER U WITH CARON
        return 468;
    if( c == 469 ) // LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON
        return 470;
    if( c == 471 ) // LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE
        return 472;
    if( c == 473 ) // LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON
        return 474;
    if( c == 475 ) // LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE
        return 476;
    if( c == 478 ) // LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON
        return 479;
    if( c == 480 ) // LATIN CAPITAL LETTER A WITH DOT ABOVE AND MACRON
        return 481;
    if( c == 482 ) // LATIN CAPITAL LETTER AE WITH MACRON
        return 483;
    if( c == 484 ) // LATIN CAPITAL LETTER G WITH STROKE
        return 485;
    if( c == 486 ) // LATIN CAPITAL LETTER G WITH CARON
        return 487;
    if( c == 488 ) // LATIN CAPITAL LETTER K WITH CARON
        return 489;
    if( c == 490 ) // LATIN CAPITAL LETTER O WITH OGONEK
        return 491;
    if( c == 492 ) // LATIN CAPITAL LETTER O WITH OGONEK AND MACRON
        return 493;
    if( c == 494 ) // LATIN CAPITAL LETTER EZH WITH CARON
        return 495;
    if( c == 497 ) // LATIN CAPITAL LETTER DZ
        return 499;
    if( c == 500 ) // LATIN CAPITAL LETTER G WITH ACUTE
        return 501;
    if( c == 502 ) // LATIN CAPITAL LETTER HWAIR
        return 405;
    if( c == 503 ) // LATIN CAPITAL LETTER WYNN
        return 447;
    if( c == 504 ) // LATIN CAPITAL LETTER N WITH GRAVE
        return 505;
    if( c == 506 ) // LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE
        return 507;
    if( c == 508 ) // LATIN CAPITAL LETTER AE WITH ACUTE
        return 509;
    if( c == 510 ) // LATIN CAPITAL LETTER O WITH STROKE AND ACUTE
        return 511;
    if( c == 512 ) // LATIN CAPITAL LETTER A WITH DOUBLE GRAVE
        return 513;
    if( c == 514 ) // LATIN CAPITAL LETTER A WITH INVERTED BREVE
        return 515;
    if( c == 516 ) // LATIN CAPITAL LETTER E WITH DOUBLE GRAVE
        return 517;
    if( c == 518 ) // LATIN CAPITAL LETTER E WITH INVERTED BREVE
        return 519;
    if( c == 520 ) // LATIN CAPITAL LETTER I WITH DOUBLE GRAVE
        return 521;
    if( c == 522 ) // LATIN CAPITAL LETTER I WITH INVERTED BREVE
        return 523;
    if( c == 524 ) // LATIN CAPITAL LETTER O WITH DOUBLE GRAVE
        return 525;
    if( c == 526 ) // LATIN CAPITAL LETTER O WITH INVERTED BREVE
        return 527;
    if( c == 528 ) // LATIN CAPITAL LETTER R WITH DOUBLE GRAVE
        return 529;
    if( c == 530 ) // LATIN CAPITAL LETTER R WITH INVERTED BREVE
        return 531;
    if( c == 532 ) // LATIN CAPITAL LETTER U WITH DOUBLE GRAVE
        return 533;
    if( c == 534 ) // LATIN CAPITAL LETTER U WITH INVERTED BREVE
        return 535;
    if( c == 536 ) // LATIN CAPITAL LETTER S WITH COMMA BELOW
        return 537;
    if( c == 538 ) // LATIN CAPITAL LETTER T WITH COMMA BELOW
        return 539;
    if( c == 540 ) // LATIN CAPITAL LETTER YOGH
        return 541;
    if( c == 542 ) // LATIN CAPITAL LETTER H WITH CARON
        return 543;
    if( c == 544 ) // LATIN CAPITAL LETTER N WITH LONG RIGHT LEG
        return 414;
    if( c == 546 ) // LATIN CAPITAL LETTER OU
        return 547;
    if( c == 548 ) // LATIN CAPITAL LETTER Z WITH HOOK
        return 549;
    if( c == 550 ) // LATIN CAPITAL LETTER A WITH DOT ABOVE
        return 551;
    if( c == 552 ) // LATIN CAPITAL LETTER E WITH CEDILLA
        return 553;
    if( c == 554 ) // LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON
        return 555;
    if( c == 556 ) // LATIN CAPITAL LETTER O WITH TILDE AND MACRON
        return 557;
    if( c == 558 ) // LATIN CAPITAL LETTER O WITH DOT ABOVE
        return 559;
    if( c == 560 ) // LATIN CAPITAL LETTER O WITH DOT ABOVE AND MACRON
        return 561;
    if( c == 562 ) // LATIN CAPITAL LETTER Y WITH MACRON
        return 563;
    if( c == 570 ) // LATIN CAPITAL LETTER A WITH STROKE
        return 11365;
    if( c == 571 ) // LATIN CAPITAL LETTER C WITH STROKE
        return 572;
    if( c == 573 ) // LATIN CAPITAL LETTER L WITH BAR
        return 410;
    if( c == 574 ) // LATIN CAPITAL LETTER T WITH DIAGONAL STROKE
        return 11366;
    if( c == 577 ) // LATIN CAPITAL LETTER GLOTTAL STOP
        return 578;
    if( c == 579 ) // LATIN CAPITAL LETTER B WITH STROKE
        return 384;
    if( c == 580 ) // LATIN CAPITAL LETTER U BAR
        return 649;
    if( c == 581 ) // LATIN CAPITAL LETTER TURNED V
        return 652;
    if( c == 582 ) // LATIN CAPITAL LETTER E WITH STROKE
        return 583;
    if( c == 584 ) // LATIN CAPITAL LETTER J WITH STROKE
        return 585;
    if( c == 586 ) // LATIN CAPITAL LETTER SMALL Q WITH HOOK TAIL
        return 587;
    if( c == 588 ) // LATIN CAPITAL LETTER R WITH STROKE
        return 589;
    if( c == 590 ) // LATIN CAPITAL LETTER Y WITH STROKE
        return 591;
    if( c == 880 ) // GREEK CAPITAL LETTER HETA
        return 881;
    if( c == 882 ) // GREEK CAPITAL LETTER ARCHAIC SAMPI
        return 883;
    if( c == 886 ) // GREEK CAPITAL LETTER PAMPHYLIAN DIGAMMA
        return 887;
    if( c == 902 ) // GREEK CAPITAL LETTER ALPHA WITH TONOS
        return 940;
    if( c >= 904 && c <= 906 ) // GREEK CAPITAL LETTER EPSILON WITH TONOS -> GREEK CAPITAL LETTER IOTA WITH TONOS
        return 941+(c-904);
    if( c == 908 ) // GREEK CAPITAL LETTER OMICRON WITH TONOS
        return 972;
    if( c >= 910 && c <= 911 ) // GREEK CAPITAL LETTER UPSILON WITH TONOS -> GREEK CAPITAL LETTER OMEGA WITH TONOS
        return 973+(c-910);
    if( c >= 913 && c <= 929 ) // GREEK CAPITAL LETTER ALPHA -> GREEK CAPITAL LETTER RHO
        return 945+(c-913);
    if( c >= 931 && c <= 939 ) // GREEK CAPITAL LETTER SIGMA -> GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA
        return 963+(c-931);
    if( c == 975 ) // GREEK CAPITAL KAI SYMBOL
        return 983;
    if( c == 984 ) // GREEK LETTER ARCHAIC KOPPA
        return 985;
    if( c == 986 ) // GREEK LETTER STIGMA
        return 987;
    if( c == 988 ) // GREEK LETTER DIGAMMA
        return 989;
    if( c == 990 ) // GREEK LETTER KOPPA
        return 991;
    if( c == 992 ) // GREEK LETTER SAMPI
        return 993;
    if( c == 994 ) // COPTIC CAPITAL LETTER SHEI
        return 995;
    if( c == 996 ) // COPTIC CAPITAL LETTER FEI
        return 997;
    if( c == 998 ) // COPTIC CAPITAL LETTER KHEI
        return 999;
    if( c == 1000 ) // COPTIC CAPITAL LETTER HORI
        return 1001;
    if( c == 1002 ) // COPTIC CAPITAL LETTER GANGIA
        return 1003;
    if( c == 1004 ) // COPTIC CAPITAL LETTER SHIMA
        return 1005;
    if( c == 1006 ) // COPTIC CAPITAL LETTER DEI
        return 1007;
    if( c == 1012 ) // GREEK CAPITAL THETA SYMBOL
        return 952;
    if( c == 1015 ) // GREEK CAPITAL LETTER SHO
        return 1016;
    if( c == 1017 ) // GREEK CAPITAL LUNATE SIGMA SYMBOL
        return 1010;
    if( c == 1018 ) // GREEK CAPITAL LETTER SAN
        return 1019;
    if( c >= 1021 && c <= 1023 ) // GREEK CAPITAL REVERSED LUNATE SIGMA SYMBOL -> GREEK CAPITAL REVERSED DOTTED LUNATE SIGMA SYMBOL
        return 891+(c-1021);
    if( c >= 1024 && c <= 1039 ) // CYRILLIC CAPITAL LETTER IE WITH GRAVE -> CYRILLIC CAPITAL LETTER DZHE
        return 1104+(c-1024);
    if( c >= 1040 && c <= 1071 ) // CYRILLIC CAPITAL LETTER A -> CYRILLIC CAPITAL LETTER YA
        return 1072+(c-1040);
    if( c == 1120 ) // CYRILLIC CAPITAL LETTER OMEGA
        return 1121;
    if( c == 1122 ) // CYRILLIC CAPITAL LETTER YAT
        return 1123;
    if( c == 1124 ) // CYRILLIC CAPITAL LETTER IOTIFIED E
        return 1125;
    if( c == 1126 ) // CYRILLIC CAPITAL LETTER LITTLE YUS
        return 1127;
    if( c == 1128 ) // CYRILLIC CAPITAL LETTER IOTIFIED LITTLE YUS
        return 1129;
    if( c == 1130 ) // CYRILLIC CAPITAL LETTER BIG YUS
        return 1131;
    if( c == 1132 ) // CYRILLIC CAPITAL LETTER IOTIFIED BIG YUS
        return 1133;
    if( c == 1134 ) // CYRILLIC CAPITAL LETTER KSI
        return 1135;
    if( c == 1136 ) // CYRILLIC CAPITAL LETTER PSI
        return 1137;
    if( c == 1138 ) // CYRILLIC CAPITAL LETTER FITA
        return 1139;
    if( c == 1140 ) // CYRILLIC CAPITAL LETTER IZHITSA
        return 1141;
    if( c == 1142 ) // CYRILLIC CAPITAL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT
        return 1143;
    if( c == 1144 ) // CYRILLIC CAPITAL LETTER UK
        return 1145;
    if( c == 1146 ) // CYRILLIC CAPITAL LETTER ROUND OMEGA
        return 1147;
    if( c == 1148 ) // CYRILLIC CAPITAL LETTER OMEGA WITH TITLO
        return 1149;
    if( c == 1150 ) // CYRILLIC CAPITAL LETTER OT
        return 1151;
    if( c == 1152 ) // CYRILLIC CAPITAL LETTER KOPPA
        return 1153;
    if( c == 1162 ) // CYRILLIC CAPITAL LETTER SHORT I WITH TAIL
        return 1163;
    if( c == 1164 ) // CYRILLIC CAPITAL LETTER SEMISOFT SIGN
        return 1165;
    if( c == 1166 ) // CYRILLIC CAPITAL LETTER ER WITH TICK
        return 1167;
    if( c == 1168 ) // CYRILLIC CAPITAL LETTER GHE WITH UPTURN
        return 1169;
    if( c == 1170 ) // CYRILLIC CAPITAL LETTER GHE WITH STROKE
        return 1171;
    if( c == 1172 ) // CYRILLIC CAPITAL LETTER GHE WITH MIDDLE HOOK
        return 1173;
    if( c == 1174 ) // CYRILLIC CAPITAL LETTER ZHE WITH DESCENDER
        return 1175;
    if( c == 1176 ) // CYRILLIC CAPITAL LETTER ZE WITH DESCENDER
        return 1177;
    if( c == 1178 ) // CYRILLIC CAPITAL LETTER KA WITH DESCENDER
        return 1179;
    if( c == 1180 ) // CYRILLIC CAPITAL LETTER KA WITH VERTICAL STROKE
        return 1181;
    if( c == 1182 ) // CYRILLIC CAPITAL LETTER KA WITH STROKE
        return 1183;
    if( c == 1184 ) // CYRILLIC CAPITAL LETTER BASHKIR KA
        return 1185;
    if( c == 1186 ) // CYRILLIC CAPITAL LETTER EN WITH DESCENDER
        return 1187;
    if( c == 1188 ) // CYRILLIC CAPITAL LIGATURE EN GHE
        return 1189;
    if( c == 1190 ) // CYRILLIC CAPITAL LETTER PE WITH MIDDLE HOOK
        return 1191;
    if( c == 1192 ) // CYRILLIC CAPITAL LETTER ABKHASIAN HA
        return 1193;
    if( c == 1194 ) // CYRILLIC CAPITAL LETTER ES WITH DESCENDER
        return 1195;
    if( c == 1196 ) // CYRILLIC CAPITAL LETTER TE WITH DESCENDER
        return 1197;
    if( c == 1198 ) // CYRILLIC CAPITAL LETTER STRAIGHT U
        return 1199;
    if( c == 1200 ) // CYRILLIC CAPITAL LETTER STRAIGHT U WITH STROKE
        return 1201;
    if( c == 1202 ) // CYRILLIC CAPITAL LETTER HA WITH DESCENDER
        return 1203;
    if( c == 1204 ) // CYRILLIC CAPITAL LIGATURE TE TSE
        return 1205;
    if( c == 1206 ) // CYRILLIC CAPITAL LETTER CHE WITH DESCENDER
        return 1207;
    if( c == 1208 ) // CYRILLIC CAPITAL LETTER CHE WITH VERTICAL STROKE
        return 1209;
    if( c == 1210 ) // CYRILLIC CAPITAL LETTER SHHA
        return 1211;
    if( c == 1212 ) // CYRILLIC CAPITAL LETTER ABKHASIAN CHE
        return 1213;
    if( c == 1214 ) // CYRILLIC CAPITAL LETTER ABKHASIAN CHE WITH DESCENDER
        return 1215;
    if( c == 1216 ) // CYRILLIC LETTER PALOCHKA
        return 1231;
    if( c == 1217 ) // CYRILLIC CAPITAL LETTER ZHE WITH BREVE
        return 1218;
    if( c == 1219 ) // CYRILLIC CAPITAL LETTER KA WITH HOOK
        return 1220;
    if( c == 1221 ) // CYRILLIC CAPITAL LETTER EL WITH TAIL
        return 1222;
    if( c == 1223 ) // CYRILLIC CAPITAL LETTER EN WITH HOOK
        return 1224;
    if( c == 1225 ) // CYRILLIC CAPITAL LETTER EN WITH TAIL
        return 1226;
    if( c == 1227 ) // CYRILLIC CAPITAL LETTER KHAKASSIAN CHE
        return 1228;
    if( c == 1229 ) // CYRILLIC CAPITAL LETTER EM WITH TAIL
        return 1230;
    if( c == 1232 ) // CYRILLIC CAPITAL LETTER A WITH BREVE
        return 1233;
    if( c == 1234 ) // CYRILLIC CAPITAL LETTER A WITH DIAERESIS
        return 1235;
    if( c == 1236 ) // CYRILLIC CAPITAL LIGATURE A IE
        return 1237;
    if( c == 1238 ) // CYRILLIC CAPITAL LETTER IE WITH BREVE
        return 1239;
    if( c == 1240 ) // CYRILLIC CAPITAL LETTER SCHWA
        return 1241;
    if( c == 1242 ) // CYRILLIC CAPITAL LETTER SCHWA WITH DIAERESIS
        return 1243;
    if( c == 1244 ) // CYRILLIC CAPITAL LETTER ZHE WITH DIAERESIS
        return 1245;
    if( c == 1246 ) // CYRILLIC CAPITAL LETTER ZE WITH DIAERESIS
        return 1247;
    if( c == 1248 ) // CYRILLIC CAPITAL LETTER ABKHASIAN DZE
        return 1249;
    if( c == 1250 ) // CYRILLIC CAPITAL LETTER I WITH MACRON
        return 1251;
    if( c == 1252 ) // CYRILLIC CAPITAL LETTER I WITH DIAERESIS
        return 1253;
    if( c == 1254 ) // CYRILLIC CAPITAL LETTER O WITH DIAERESIS
        return 1255;
    if( c == 1256 ) // CYRILLIC CAPITAL LETTER BARRED O
        return 1257;
    if( c == 1258 ) // CYRILLIC CAPITAL LETTER BARRED O WITH DIAERESIS
        return 1259;
    if( c == 1260 ) // CYRILLIC CAPITAL LETTER E WITH DIAERESIS
        return 1261;
    if( c == 1262 ) // CYRILLIC CAPITAL LETTER U WITH MACRON
        return 1263;
    if( c == 1264 ) // CYRILLIC CAPITAL LETTER U WITH DIAERESIS
        return 1265;
    if( c == 1266 ) // CYRILLIC CAPITAL LETTER U WITH DOUBLE ACUTE
        return 1267;
    if( c == 1268 ) // CYRILLIC CAPITAL LETTER CHE WITH DIAERESIS
        return 1269;
    if( c == 1270 ) // CYRILLIC CAPITAL LETTER GHE WITH DESCENDER
        return 1271;
    if( c == 1272 ) // CYRILLIC CAPITAL LETTER YERU WITH DIAERESIS
        return 1273;
    if( c == 1274 ) // CYRILLIC CAPITAL LETTER GHE WITH STROKE AND HOOK
        return 1275;
    if( c == 1276 ) // CYRILLIC CAPITAL LETTER HA WITH HOOK
        return 1277;
    if( c == 1278 ) // CYRILLIC CAPITAL LETTER HA WITH STROKE
        return 1279;
    if( c == 1280 ) // CYRILLIC CAPITAL LETTER KOMI DE
        return 1281;
    if( c == 1282 ) // CYRILLIC CAPITAL LETTER KOMI DJE
        return 1283;
    if( c == 1284 ) // CYRILLIC CAPITAL LETTER KOMI ZJE
        return 1285;
    if( c == 1286 ) // CYRILLIC CAPITAL LETTER KOMI DZJE
        return 1287;
    if( c == 1288 ) // CYRILLIC CAPITAL LETTER KOMI LJE
        return 1289;
    if( c == 1290 ) // CYRILLIC CAPITAL LETTER KOMI NJE
        return 1291;
    if( c == 1292 ) // CYRILLIC CAPITAL LETTER KOMI SJE
        return 1293;
    if( c == 1294 ) // CYRILLIC CAPITAL LETTER KOMI TJE
        return 1295;
    if( c == 1296 ) // CYRILLIC CAPITAL LETTER REVERSED ZE
        return 1297;
    if( c == 1298 ) // CYRILLIC CAPITAL LETTER EL WITH HOOK
        return 1299;
    if( c == 1300 ) // CYRILLIC CAPITAL LETTER LHA
        return 1301;
    if( c == 1302 ) // CYRILLIC CAPITAL LETTER RHA
        return 1303;
    if( c == 1304 ) // CYRILLIC CAPITAL LETTER YAE
        return 1305;
    if( c == 1306 ) // CYRILLIC CAPITAL LETTER QA
        return 1307;
    if( c == 1308 ) // CYRILLIC CAPITAL LETTER WE
        return 1309;
    if( c == 1310 ) // CYRILLIC CAPITAL LETTER ALEUT KA
        return 1311;
    if( c == 1312 ) // CYRILLIC CAPITAL LETTER EL WITH MIDDLE HOOK
        return 1313;
    if( c == 1314 ) // CYRILLIC CAPITAL LETTER EN WITH MIDDLE HOOK
        return 1315;
    if( c >= 1329 && c <= 1366 ) // ARMENIAN CAPITAL LETTER AYB -> ARMENIAN CAPITAL LETTER FEH
        return 1377+(c-1329);
    if( c >= 4256 && c <= 4293 ) // GEORGIAN CAPITAL LETTER AN -> GEORGIAN CAPITAL LETTER HOE
        return 11520+(c-4256);
    if( c == 7680 ) // LATIN CAPITAL LETTER A WITH RING BELOW
        return 7681;
    if( c == 7682 ) // LATIN CAPITAL LETTER B WITH DOT ABOVE
        return 7683;
    if( c == 7684 ) // LATIN CAPITAL LETTER B WITH DOT BELOW
        return 7685;
    if( c == 7686 ) // LATIN CAPITAL LETTER B WITH LINE BELOW
        return 7687;
    if( c == 7688 ) // LATIN CAPITAL LETTER C WITH CEDILLA AND ACUTE
        return 7689;
    if( c == 7690 ) // LATIN CAPITAL LETTER D WITH DOT ABOVE
        return 7691;
    if( c == 7692 ) // LATIN CAPITAL LETTER D WITH DOT BELOW
        return 7693;
    if( c == 7694 ) // LATIN CAPITAL LETTER D WITH LINE BELOW
        return 7695;
    if( c == 7696 ) // LATIN CAPITAL LETTER D WITH CEDILLA
        return 7697;
    if( c == 7698 ) // LATIN CAPITAL LETTER D WITH CIRCUMFLEX BELOW
        return 7699;
    if( c == 7700 ) // LATIN CAPITAL LETTER E WITH MACRON AND GRAVE
        return 7701;
    if( c == 7702 ) // LATIN CAPITAL LETTER E WITH MACRON AND ACUTE
        return 7703;
    if( c == 7704 ) // LATIN CAPITAL LETTER E WITH CIRCUMFLEX BELOW
        return 7705;
    if( c == 7706 ) // LATIN CAPITAL LETTER E WITH TILDE BELOW
        return 7707;
    if( c == 7708 ) // LATIN CAPITAL LETTER E WITH CEDILLA AND BREVE
        return 7709;
    if( c == 7710 ) // LATIN CAPITAL LETTER F WITH DOT ABOVE
        return 7711;
    if( c == 7712 ) // LATIN CAPITAL LETTER G WITH MACRON
        return 7713;
    if( c == 7714 ) // LATIN CAPITAL LETTER H WITH DOT ABOVE
        return 7715;
    if( c == 7716 ) // LATIN CAPITAL LETTER H WITH DOT BELOW
        return 7717;
    if( c == 7718 ) // LATIN CAPITAL LETTER H WITH DIAERESIS
        return 7719;
    if( c == 7720 ) // LATIN CAPITAL LETTER H WITH CEDILLA
        return 7721;
    if( c == 7722 ) // LATIN CAPITAL LETTER H WITH BREVE BELOW
        return 7723;
    if( c == 7724 ) // LATIN CAPITAL LETTER I WITH TILDE BELOW
        return 7725;
    if( c == 7726 ) // LATIN CAPITAL LETTER I WITH DIAERESIS AND ACUTE
        return 7727;
    if( c == 7728 ) // LATIN CAPITAL LETTER K WITH ACUTE
        return 7729;
    if( c == 7730 ) // LATIN CAPITAL LETTER K WITH DOT BELOW
        return 7731;
    if( c == 7732 ) // LATIN CAPITAL LETTER K WITH LINE BELOW
        return 7733;
    if( c == 7734 ) // LATIN CAPITAL LETTER L WITH DOT BELOW
        return 7735;
    if( c == 7736 ) // LATIN CAPITAL LETTER L WITH DOT BELOW AND MACRON
        return 7737;
    if( c == 7738 ) // LATIN CAPITAL LETTER L WITH LINE BELOW
        return 7739;
    if( c == 7740 ) // LATIN CAPITAL LETTER L WITH CIRCUMFLEX BELOW
        return 7741;
    if( c == 7742 ) // LATIN CAPITAL LETTER M WITH ACUTE
        return 7743;
    if( c == 7744 ) // LATIN CAPITAL LETTER M WITH DOT ABOVE
        return 7745;
    if( c == 7746 ) // LATIN CAPITAL LETTER M WITH DOT BELOW
        return 7747;
    if( c == 7748 ) // LATIN CAPITAL LETTER N WITH DOT ABOVE
        return 7749;
    if( c == 7750 ) // LATIN CAPITAL LETTER N WITH DOT BELOW
        return 7751;
    if( c == 7752 ) // LATIN CAPITAL LETTER N WITH LINE BELOW
        return 7753;
    if( c == 7754 ) // LATIN CAPITAL LETTER N WITH CIRCUMFLEX BELOW
        return 7755;
    if( c == 7756 ) // LATIN CAPITAL LETTER O WITH TILDE AND ACUTE
        return 7757;
    if( c == 7758 ) // LATIN CAPITAL LETTER O WITH TILDE AND DIAERESIS
        return 7759;
    if( c == 7760 ) // LATIN CAPITAL LETTER O WITH MACRON AND GRAVE
        return 7761;
    if( c == 7762 ) // LATIN CAPITAL LETTER O WITH MACRON AND ACUTE
        return 7763;
    if( c == 7764 ) // LATIN CAPITAL LETTER P WITH ACUTE
        return 7765;
    if( c == 7766 ) // LATIN CAPITAL LETTER P WITH DOT ABOVE
        return 7767;
    if( c == 7768 ) // LATIN CAPITAL LETTER R WITH DOT ABOVE
        return 7769;
    if( c == 7770 ) // LATIN CAPITAL LETTER R WITH DOT BELOW
        return 7771;
    if( c == 7772 ) // LATIN CAPITAL LETTER R WITH DOT BELOW AND MACRON
        return 7773;
    if( c == 7774 ) // LATIN CAPITAL LETTER R WITH LINE BELOW
        return 7775;
    if( c == 7776 ) // LATIN CAPITAL LETTER S WITH DOT ABOVE
        return 7777;
    if( c == 7778 ) // LATIN CAPITAL LETTER S WITH DOT BELOW
        return 7779;
    if( c == 7780 ) // LATIN CAPITAL LETTER S WITH ACUTE AND DOT ABOVE
        return 7781;
    if( c == 7782 ) // LATIN CAPITAL LETTER S WITH CARON AND DOT ABOVE
        return 7783;
    if( c == 7784 ) // LATIN CAPITAL LETTER S WITH DOT BELOW AND DOT ABOVE
        return 7785;
    if( c == 7786 ) // LATIN CAPITAL LETTER T WITH DOT ABOVE
        return 7787;
    if( c == 7788 ) // LATIN CAPITAL LETTER T WITH DOT BELOW
        return 7789;
    if( c == 7790 ) // LATIN CAPITAL LETTER T WITH LINE BELOW
        return 7791;
    if( c == 7792 ) // LATIN CAPITAL LETTER T WITH CIRCUMFLEX BELOW
        return 7793;
    if( c == 7794 ) // LATIN CAPITAL LETTER U WITH DIAERESIS BELOW
        return 7795;
    if( c == 7796 ) // LATIN CAPITAL LETTER U WITH TILDE BELOW
        return 7797;
    if( c == 7798 ) // LATIN CAPITAL LETTER U WITH CIRCUMFLEX BELOW
        return 7799;
    if( c == 7800 ) // LATIN CAPITAL LETTER U WITH TILDE AND ACUTE
        return 7801;
    if( c == 7802 ) // LATIN CAPITAL LETTER U WITH MACRON AND DIAERESIS
        return 7803;
    if( c == 7804 ) // LATIN CAPITAL LETTER V WITH TILDE
        return 7805;
    if( c == 7806 ) // LATIN CAPITAL LETTER V WITH DOT BELOW
        return 7807;
    if( c == 7808 ) // LATIN CAPITAL LETTER W WITH GRAVE
        return 7809;
    if( c == 7810 ) // LATIN CAPITAL LETTER W WITH ACUTE
        return 7811;
    if( c == 7812 ) // LATIN CAPITAL LETTER W WITH DIAERESIS
        return 7813;
    if( c == 7814 ) // LATIN CAPITAL LETTER W WITH DOT ABOVE
        return 7815;
    if( c == 7816 ) // LATIN CAPITAL LETTER W WITH DOT BELOW
        return 7817;
    if( c == 7818 ) // LATIN CAPITAL LETTER X WITH DOT ABOVE
        return 7819;
    if( c == 7820 ) // LATIN CAPITAL LETTER X WITH DIAERESIS
        return 7821;
    if( c == 7822 ) // LATIN CAPITAL LETTER Y WITH DOT ABOVE
        return 7823;
    if( c == 7824 ) // LATIN CAPITAL LETTER Z WITH CIRCUMFLEX
        return 7825;
    if( c == 7826 ) // LATIN CAPITAL LETTER Z WITH DOT BELOW
        return 7827;
    if( c == 7828 ) // LATIN CAPITAL LETTER Z WITH LINE BELOW
        return 7829;
    if( c == 7838 ) // LATIN CAPITAL LETTER SHARP S
        return 223;
    if( c == 7840 ) // LATIN CAPITAL LETTER A WITH DOT BELOW
        return 7841;
    if( c == 7842 ) // LATIN CAPITAL LETTER A WITH HOOK ABOVE
        return 7843;
    if( c == 7844 ) // LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND ACUTE
        return 7845;
    if( c == 7846 ) // LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND GRAVE
        return 7847;
    if( c == 7848 ) // LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE
        return 7849;
    if( c == 7850 ) // LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND TILDE
        return 7851;
    if( c == 7852 ) // LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND DOT BELOW
        return 7853;
    if( c == 7854 ) // LATIN CAPITAL LETTER A WITH BREVE AND ACUTE
        return 7855;
    if( c == 7856 ) // LATIN CAPITAL LETTER A WITH BREVE AND GRAVE
        return 7857;
    if( c == 7858 ) // LATIN CAPITAL LETTER A WITH BREVE AND HOOK ABOVE
        return 7859;
    if( c == 7860 ) // LATIN CAPITAL LETTER A WITH BREVE AND TILDE
        return 7861;
    if( c == 7862 ) // LATIN CAPITAL LETTER A WITH BREVE AND DOT BELOW
        return 7863;
    if( c == 7864 ) // LATIN CAPITAL LETTER E WITH DOT BELOW
        return 7865;
    if( c == 7866 ) // LATIN CAPITAL LETTER E WITH HOOK ABOVE
        return 7867;
    if( c == 7868 ) // LATIN CAPITAL LETTER E WITH TILDE
        return 7869;
    if( c == 7870 ) // LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND ACUTE
        return 7871;
    if( c == 7872 ) // LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND GRAVE
        return 7873;
    if( c == 7874 ) // LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE
        return 7875;
    if( c == 7876 ) // LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND TILDE
        return 7877;
    if( c == 7878 ) // LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND DOT BELOW
        return 7879;
    if( c == 7880 ) // LATIN CAPITAL LETTER I WITH HOOK ABOVE
        return 7881;
    if( c == 7882 ) // LATIN CAPITAL LETTER I WITH DOT BELOW
        return 7883;
    if( c == 7884 ) // LATIN CAPITAL LETTER O WITH DOT BELOW
        return 7885;
    if( c == 7886 ) // LATIN CAPITAL LETTER O WITH HOOK ABOVE
        return 7887;
    if( c == 7888 ) // LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND ACUTE
        return 7889;
    if( c == 7890 ) // LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND GRAVE
        return 7891;
    if( c == 7892 ) // LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE
        return 7893;
    if( c == 7894 ) // LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND TILDE
        return 7895;
    if( c == 7896 ) // LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW
        return 7897;
    if( c == 7898 ) // LATIN CAPITAL LETTER O WITH HORN AND ACUTE
        return 7899;
    if( c == 7900 ) // LATIN CAPITAL LETTER O WITH HORN AND GRAVE
        return 7901;
    if( c == 7902 ) // LATIN CAPITAL LETTER O WITH HORN AND HOOK ABOVE
        return 7903;
    if( c == 7904 ) // LATIN CAPITAL LETTER O WITH HORN AND TILDE
        return 7905;
    if( c == 7906 ) // LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW
        return 7907;
    if( c == 7908 ) // LATIN CAPITAL LETTER U WITH DOT BELOW
        return 7909;
    if( c == 7910 ) // LATIN CAPITAL LETTER U WITH HOOK ABOVE
        return 7911;
    if( c == 7912 ) // LATIN CAPITAL LETTER U WITH HORN AND ACUTE
        return 7913;
    if( c == 7914 ) // LATIN CAPITAL LETTER U WITH HORN AND GRAVE
        return 7915;
    if( c == 7916 ) // LATIN CAPITAL LETTER U WITH HORN AND HOOK ABOVE
        return 7917;
    if( c == 7918 ) // LATIN CAPITAL LETTER U WITH HORN AND TILDE
        return 7919;
    if( c == 7920 ) // LATIN CAPITAL LETTER U WITH HORN AND DOT BELOW
        return 7921;
    if( c == 7922 ) // LATIN CAPITAL LETTER Y WITH GRAVE
        return 7923;
    if( c == 7924 ) // LATIN CAPITAL LETTER Y WITH DOT BELOW
        return 7925;
    if( c == 7926 ) // LATIN CAPITAL LETTER Y WITH HOOK ABOVE
        return 7927;
    if( c == 7928 ) // LATIN CAPITAL LETTER Y WITH TILDE
        return 7929;
    if( c == 7930 ) // LATIN CAPITAL LETTER MIDDLE-WELSH LL
        return 7931;
    if( c == 7932 ) // LATIN CAPITAL LETTER MIDDLE-WELSH V
        return 7933;
    if( c == 7934 ) // LATIN CAPITAL LETTER Y WITH LOOP
        return 7935;
    if( c >= 7944 && c <= 7951 ) // GREEK CAPITAL LETTER ALPHA WITH PSILI -> GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI
        return 7936+(c-7944);
    if( c >= 7960 && c <= 7965 ) // GREEK CAPITAL LETTER EPSILON WITH PSILI -> GREEK CAPITAL LETTER EPSILON WITH DASIA AND OXIA
        return 7952+(c-7960);
    if( c >= 7976 && c <= 7983 ) // GREEK CAPITAL LETTER ETA WITH PSILI -> GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI
        return 7968+(c-7976);
    if( c >= 7992 && c <= 7999 ) // GREEK CAPITAL LETTER IOTA WITH PSILI -> GREEK CAPITAL LETTER IOTA WITH DASIA AND PERISPOMENI
        return 7984+(c-7992);
    if( c >= 8008 && c <= 8013 ) // GREEK CAPITAL LETTER OMICRON WITH PSILI -> GREEK CAPITAL LETTER OMICRON WITH DASIA AND OXIA
        return 8000+(c-8008);
    if( c == 8025 ) // GREEK CAPITAL LETTER UPSILON WITH DASIA
        return 8017;
    if( c == 8027 ) // GREEK CAPITAL LETTER UPSILON WITH DASIA AND VARIA
        return 8019;
    if( c == 8029 ) // GREEK CAPITAL LETTER UPSILON WITH DASIA AND OXIA
        return 8021;
    if( c == 8031 ) // GREEK CAPITAL LETTER UPSILON WITH DASIA AND PERISPOMENI
        return 8023;
    if( c >= 8040 && c <= 8047 ) // GREEK CAPITAL LETTER OMEGA WITH PSILI -> GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI
        return 8032+(c-8040);
    if( c >= 8120 && c <= 8121 ) // GREEK CAPITAL LETTER ALPHA WITH VRACHY -> GREEK CAPITAL LETTER ALPHA WITH MACRON
        return 8112+(c-8120);
    if( c >= 8122 && c <= 8123 ) // GREEK CAPITAL LETTER ALPHA WITH VARIA -> GREEK CAPITAL LETTER ALPHA WITH OXIA
        return 8048+(c-8122);
    if( c >= 8136 && c <= 8139 ) // GREEK CAPITAL LETTER EPSILON WITH VARIA -> GREEK CAPITAL LETTER ETA WITH OXIA
        return 8050+(c-8136);
    if( c >= 8152 && c <= 8153 ) // GREEK CAPITAL LETTER IOTA WITH VRACHY -> GREEK CAPITAL LETTER IOTA WITH MACRON
        return 8144+(c-8152);
    if( c >= 8154 && c <= 8155 ) // GREEK CAPITAL LETTER IOTA WITH VARIA -> GREEK CAPITAL LETTER IOTA WITH OXIA
        return 8054+(c-8154);
    if( c >= 8168 && c <= 8169 ) // GREEK CAPITAL LETTER UPSILON WITH VRACHY -> GREEK CAPITAL LETTER UPSILON WITH MACRON
        return 8160+(c-8168);
    if( c >= 8170 && c <= 8171 ) // GREEK CAPITAL LETTER UPSILON WITH VARIA -> GREEK CAPITAL LETTER UPSILON WITH OXIA
        return 8058+(c-8170);
    if( c == 8172 ) // GREEK CAPITAL LETTER RHO WITH DASIA
        return 8165;
    if( c >= 8184 && c <= 8185 ) // GREEK CAPITAL LETTER OMICRON WITH VARIA -> GREEK CAPITAL LETTER OMICRON WITH OXIA
        return 8056+(c-8184);
    if( c >= 8186 && c <= 8187 ) // GREEK CAPITAL LETTER OMEGA WITH VARIA -> GREEK CAPITAL LETTER OMEGA WITH OXIA
        return 8060+(c-8186);
    if( c == 8486 ) // OHM SIGN
        return 969;
    if( c == 8490 ) // KELVIN SIGN
        return 107;
    if( c == 8491 ) // ANGSTROM SIGN
        return 229;
    if( c == 8498 ) // TURNED CAPITAL F
        return 8526;
    if( c == 8579 ) // ROMAN NUMERAL REVERSED ONE HUNDRED
        return 8580;
    if( c >= 11264 && c <= 11310 ) // GLAGOLITIC CAPITAL LETTER AZU -> GLAGOLITIC CAPITAL LETTER LATINATE MYSLITE
        return 11312+(c-11264);
    if( c == 11360 ) // LATIN CAPITAL LETTER L WITH DOUBLE BAR
        return 11361;
    if( c == 11362 ) // LATIN CAPITAL LETTER L WITH MIDDLE TILDE
        return 619;
    if( c == 11363 ) // LATIN CAPITAL LETTER P WITH STROKE
        return 7549;
    if( c == 11364 ) // LATIN CAPITAL LETTER R WITH TAIL
        return 637;
    if( c == 11367 ) // LATIN CAPITAL LETTER H WITH DESCENDER
        return 11368;
    if( c == 11369 ) // LATIN CAPITAL LETTER K WITH DESCENDER
        return 11370;
    if( c == 11371 ) // LATIN CAPITAL LETTER Z WITH DESCENDER
        return 11372;
    if( c == 11373 ) // LATIN CAPITAL LETTER ALPHA
        return 593;
    if( c == 11374 ) // LATIN CAPITAL LETTER M WITH HOOK
        return 625;
    if( c == 11375 ) // LATIN CAPITAL LETTER TURNED A
        return 592;
    if( c == 11378 ) // LATIN CAPITAL LETTER W WITH HOOK
        return 11379;
    if( c == 11381 ) // LATIN CAPITAL LETTER HALF H
        return 11382;
    if( c == 11392 ) // COPTIC CAPITAL LETTER ALFA
        return 11393;
    if( c == 11394 ) // COPTIC CAPITAL LETTER VIDA
        return 11395;
    if( c == 11396 ) // COPTIC CAPITAL LETTER GAMMA
        return 11397;
    if( c == 11398 ) // COPTIC CAPITAL LETTER DALDA
        return 11399;
    if( c == 11400 ) // COPTIC CAPITAL LETTER EIE
        return 11401;
    if( c == 11402 ) // COPTIC CAPITAL LETTER SOU
        return 11403;
    if( c == 11404 ) // COPTIC CAPITAL LETTER ZATA
        return 11405;
    if( c == 11406 ) // COPTIC CAPITAL LETTER HATE
        return 11407;
    if( c == 11408 ) // COPTIC CAPITAL LETTER THETHE
        return 11409;
    if( c == 11410 ) // COPTIC CAPITAL LETTER IAUDA
        return 11411;
    if( c == 11412 ) // COPTIC CAPITAL LETTER KAPA
        return 11413;
    if( c == 11414 ) // COPTIC CAPITAL LETTER LAULA
        return 11415;
    if( c == 11416 ) // COPTIC CAPITAL LETTER MI
        return 11417;
    if( c == 11418 ) // COPTIC CAPITAL LETTER NI
        return 11419;
    if( c == 11420 ) // COPTIC CAPITAL LETTER KSI
        return 11421;
    if( c == 11422 ) // COPTIC CAPITAL LETTER O
        return 11423;
    if( c == 11424 ) // COPTIC CAPITAL LETTER PI
        return 11425;
    if( c == 11426 ) // COPTIC CAPITAL LETTER RO
        return 11427;
    if( c == 11428 ) // COPTIC CAPITAL LETTER SIMA
        return 11429;
    if( c == 11430 ) // COPTIC CAPITAL LETTER TAU
        return 11431;
    if( c == 11432 ) // COPTIC CAPITAL LETTER UA
        return 11433;
    if( c == 11434 ) // COPTIC CAPITAL LETTER FI
        return 11435;
    if( c == 11436 ) // COPTIC CAPITAL LETTER KHI
        return 11437;
    if( c == 11438 ) // COPTIC CAPITAL LETTER PSI
        return 11439;
    if( c == 11440 ) // COPTIC CAPITAL LETTER OOU
        return 11441;
    if( c == 11442 ) // COPTIC CAPITAL LETTER DIALECT-P ALEF
        return 11443;
    if( c == 11444 ) // COPTIC CAPITAL LETTER OLD COPTIC AIN
        return 11445;
    if( c == 11446 ) // COPTIC CAPITAL LETTER CRYPTOGRAMMIC EIE
        return 11447;
    if( c == 11448 ) // COPTIC CAPITAL LETTER DIALECT-P KAPA
        return 11449;
    if( c == 11450 ) // COPTIC CAPITAL LETTER DIALECT-P NI
        return 11451;
    if( c == 11452 ) // COPTIC CAPITAL LETTER CRYPTOGRAMMIC NI
        return 11453;
    if( c == 11454 ) // COPTIC CAPITAL LETTER OLD COPTIC OOU
        return 11455;
    if( c == 11456 ) // COPTIC CAPITAL LETTER SAMPI
        return 11457;
    if( c == 11458 ) // COPTIC CAPITAL LETTER CROSSED SHEI
        return 11459;
    if( c == 11460 ) // COPTIC CAPITAL LETTER OLD COPTIC SHEI
        return 11461;
    if( c == 11462 ) // COPTIC CAPITAL LETTER OLD COPTIC ESH
        return 11463;
    if( c == 11464 ) // COPTIC CAPITAL LETTER AKHMIMIC KHEI
        return 11465;
    if( c == 11466 ) // COPTIC CAPITAL LETTER DIALECT-P HORI
        return 11467;
    if( c == 11468 ) // COPTIC CAPITAL LETTER OLD COPTIC HORI
        return 11469;
    if( c == 11470 ) // COPTIC CAPITAL LETTER OLD COPTIC HA
        return 11471;
    if( c == 11472 ) // COPTIC CAPITAL LETTER L-SHAPED HA
        return 11473;
    if( c == 11474 ) // COPTIC CAPITAL LETTER OLD COPTIC HEI
        return 11475;
    if( c == 11476 ) // COPTIC CAPITAL LETTER OLD COPTIC HAT
        return 11477;
    if( c == 11478 ) // COPTIC CAPITAL LETTER OLD COPTIC GANGIA
        return 11479;
    if( c == 11480 ) // COPTIC CAPITAL LETTER OLD COPTIC DJA
        return 11481;
    if( c == 11482 ) // COPTIC CAPITAL LETTER OLD COPTIC SHIMA
        return 11483;
    if( c == 11484 ) // COPTIC CAPITAL LETTER OLD NUBIAN SHIMA
        return 11485;
    if( c == 11486 ) // COPTIC CAPITAL LETTER OLD NUBIAN NGI
        return 11487;
    if( c == 11488 ) // COPTIC CAPITAL LETTER OLD NUBIAN NYI
        return 11489;
    if( c == 11490 ) // COPTIC CAPITAL LETTER OLD NUBIAN WAU
        return 11491;
    if( c == 42560 ) // CYRILLIC CAPITAL LETTER ZEMLYA
        return 42561;
    if( c == 42562 ) // CYRILLIC CAPITAL LETTER DZELO
        return 42563;
    if( c == 42564 ) // CYRILLIC CAPITAL LETTER REVERSED DZE
        return 42565;
    if( c == 42566 ) // CYRILLIC CAPITAL LETTER IOTA
        return 42567;
    if( c == 42568 ) // CYRILLIC CAPITAL LETTER DJERV
        return 42569;
    if( c == 42570 ) // CYRILLIC CAPITAL LETTER MONOGRAPH UK
        return 42571;
    if( c == 42572 ) // CYRILLIC CAPITAL LETTER BROAD OMEGA
        return 42573;
    if( c == 42574 ) // CYRILLIC CAPITAL LETTER NEUTRAL YER
        return 42575;
    if( c == 42576 ) // CYRILLIC CAPITAL LETTER YERU WITH BACK YER
        return 42577;
    if( c == 42578 ) // CYRILLIC CAPITAL LETTER IOTIFIED YAT
        return 42579;
    if( c == 42580 ) // CYRILLIC CAPITAL LETTER REVERSED YU
        return 42581;
    if( c == 42582 ) // CYRILLIC CAPITAL LETTER IOTIFIED A
        return 42583;
    if( c == 42584 ) // CYRILLIC CAPITAL LETTER CLOSED LITTLE YUS
        return 42585;
    if( c == 42586 ) // CYRILLIC CAPITAL LETTER BLENDED YUS
        return 42587;
    if( c == 42588 ) // CYRILLIC CAPITAL LETTER IOTIFIED CLOSED LITTLE YUS
        return 42589;
    if( c == 42590 ) // CYRILLIC CAPITAL LETTER YN
        return 42591;
    if( c == 42594 ) // CYRILLIC CAPITAL LETTER SOFT DE
        return 42595;
    if( c == 42596 ) // CYRILLIC CAPITAL LETTER SOFT EL
        return 42597;
    if( c == 42598 ) // CYRILLIC CAPITAL LETTER SOFT EM
        return 42599;
    if( c == 42600 ) // CYRILLIC CAPITAL LETTER MONOCULAR O
        return 42601;
    if( c == 42602 ) // CYRILLIC CAPITAL LETTER BINOCULAR O
        return 42603;
    if( c == 42604 ) // CYRILLIC CAPITAL LETTER DOUBLE MONOCULAR O
        return 42605;
    if( c == 42624 ) // CYRILLIC CAPITAL LETTER DWE
        return 42625;
    if( c == 42626 ) // CYRILLIC CAPITAL LETTER DZWE
        return 42627;
    if( c == 42628 ) // CYRILLIC CAPITAL LETTER ZHWE
        return 42629;
    if( c == 42630 ) // CYRILLIC CAPITAL LETTER CCHE
        return 42631;
    if( c == 42632 ) // CYRILLIC CAPITAL LETTER DZZE
        return 42633;
    if( c == 42634 ) // CYRILLIC CAPITAL LETTER TE WITH MIDDLE HOOK
        return 42635;
    if( c == 42636 ) // CYRILLIC CAPITAL LETTER TWE
        return 42637;
    if( c == 42638 ) // CYRILLIC CAPITAL LETTER TSWE
        return 42639;
    if( c == 42640 ) // CYRILLIC CAPITAL LETTER TSSE
        return 42641;
    if( c == 42642 ) // CYRILLIC CAPITAL LETTER TCHE
        return 42643;
    if( c == 42644 ) // CYRILLIC CAPITAL LETTER HWE
        return 42645;
    if( c == 42646 ) // CYRILLIC CAPITAL LETTER SHWE
        return 42647;
    if( c == 42786 ) // LATIN CAPITAL LETTER EGYPTOLOGICAL ALEF
        return 42787;
    if( c == 42788 ) // LATIN CAPITAL LETTER EGYPTOLOGICAL AIN
        return 42789;
    if( c == 42790 ) // LATIN CAPITAL LETTER HENG
        return 42791;
    if( c == 42792 ) // LATIN CAPITAL LETTER TZ
        return 42793;
    if( c == 42794 ) // LATIN CAPITAL LETTER TRESILLO
        return 42795;
    if( c == 42796 ) // LATIN CAPITAL LETTER CUATRILLO
        return 42797;
    if( c == 42798 ) // LATIN CAPITAL LETTER CUATRILLO WITH COMMA
        return 42799;
    if( c == 42802 ) // LATIN CAPITAL LETTER AA
        return 42803;
    if( c == 42804 ) // LATIN CAPITAL LETTER AO
        return 42805;
    if( c == 42806 ) // LATIN CAPITAL LETTER AU
        return 42807;
    if( c == 42808 ) // LATIN CAPITAL LETTER AV
        return 42809;
    if( c == 42810 ) // LATIN CAPITAL LETTER AV WITH HORIZONTAL BAR
        return 42811;
    if( c == 42812 ) // LATIN CAPITAL LETTER AY
        return 42813;
    if( c == 42814 ) // LATIN CAPITAL LETTER REVERSED C WITH DOT
        return 42815;
    if( c == 42816 ) // LATIN CAPITAL LETTER K WITH STROKE
        return 42817;
    if( c == 42818 ) // LATIN CAPITAL LETTER K WITH DIAGONAL STROKE
        return 42819;
    if( c == 42820 ) // LATIN CAPITAL LETTER K WITH STROKE AND DIAGONAL STROKE
        return 42821;
    if( c == 42822 ) // LATIN CAPITAL LETTER BROKEN L
        return 42823;
    if( c == 42824 ) // LATIN CAPITAL LETTER L WITH HIGH STROKE
        return 42825;
    if( c == 42826 ) // LATIN CAPITAL LETTER O WITH LONG STROKE OVERLAY
        return 42827;
    if( c == 42828 ) // LATIN CAPITAL LETTER O WITH LOOP
        return 42829;
    if( c == 42830 ) // LATIN CAPITAL LETTER OO
        return 42831;
    if( c == 42832 ) // LATIN CAPITAL LETTER P WITH STROKE THROUGH DESCENDER
        return 42833;
    if( c == 42834 ) // LATIN CAPITAL LETTER P WITH FLOURISH
        return 42835;
    if( c == 42836 ) // LATIN CAPITAL LETTER P WITH SQUIRREL TAIL
        return 42837;
    if( c == 42838 ) // LATIN CAPITAL LETTER Q WITH STROKE THROUGH DESCENDER
        return 42839;
    if( c == 42840 ) // LATIN CAPITAL LETTER Q WITH DIAGONAL STROKE
        return 42841;
    if( c == 42842 ) // LATIN CAPITAL LETTER R ROTUNDA
        return 42843;
    if( c == 42844 ) // LATIN CAPITAL LETTER RUM ROTUNDA
        return 42845;
    if( c == 42846 ) // LATIN CAPITAL LETTER V WITH DIAGONAL STROKE
        return 42847;
    if( c == 42848 ) // LATIN CAPITAL LETTER VY
        return 42849;
    if( c == 42850 ) // LATIN CAPITAL LETTER VISIGOTHIC Z
        return 42851;
    if( c == 42852 ) // LATIN CAPITAL LETTER THORN WITH STROKE
        return 42853;
    if( c == 42854 ) // LATIN CAPITAL LETTER THORN WITH STROKE THROUGH DESCENDER
        return 42855;
    if( c == 42856 ) // LATIN CAPITAL LETTER VEND
        return 42857;
    if( c == 42858 ) // LATIN CAPITAL LETTER ET
        return 42859;
    if( c == 42860 ) // LATIN CAPITAL LETTER IS
        return 42861;
    if( c == 42862 ) // LATIN CAPITAL LETTER CON
        return 42863;
    if( c == 42873 ) // LATIN CAPITAL LETTER INSULAR D
        return 42874;
    if( c == 42875 ) // LATIN CAPITAL LETTER INSULAR F
        return 42876;
    if( c == 42877 ) // LATIN CAPITAL LETTER INSULAR G
        return 7545;
    if( c == 42878 ) // LATIN CAPITAL LETTER TURNED INSULAR G
        return 42879;
    if( c == 42880 ) // LATIN CAPITAL LETTER TURNED L
        return 42881;
    if( c == 42882 ) // LATIN CAPITAL LETTER INSULAR R
        return 42883;
    if( c == 42884 ) // LATIN CAPITAL LETTER INSULAR S
        return 42885;
    if( c == 42886 ) // LATIN CAPITAL LETTER INSULAR T
        return 42887;
    if( c == 42891 ) // LATIN CAPITAL LETTER SALTILLO
        return 42892;
    if( c >= 65313 && c <= 65338 ) // FULLWIDTH LATIN CAPITAL LETTER A -> FULLWIDTH LATIN CAPITAL LETTER Z
        return 65345+(c-65313);
    if( c >= 66560 && c <= 66599 ) // DESERET CAPITAL LETTER LONG I -> DESERET CAPITAL LETTER EW
        return 66600+(c-66560);
    return c;
}

