// Copyright (c) 2008-2011 Mighty Heave Software LLC.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#ifndef __rx_h
#define __rx_h

#define RX_OPEN         (0)
#define RX_CASEI        (1)
#define RX_CLOSE        (0)

#include "glrexport.h"
#include "intset.h"
#include "str.h"

class Nfa;

class LIBGLR_EXPORT Rx
{
public:
    enum RxType { Concat, Union, Star, Plus, Opt, Many, Chars, Capture, Unknown };
    Rx();
    virtual ~Rx();
    virtual Rx *Copy() = 0;
    virtual RxType RxClassification() { return Rx::Unknown; }
    virtual void CaseDesensitize() = 0;
    virtual int AddToNfa(Nfa &nfa, int startstate) = 0;
    virtual bool EqualTo(Rx *rhs) = 0;
};

class LIBGLR_EXPORT RxConcat : public Rx
{
public:
    Rx *m_lhs, *m_rhs;
    RxConcat(Rx *lhs, Rx *rhs);
    virtual ~RxConcat();
    virtual Rx *Copy();
    virtual RxType RxClassification() { return Rx::Concat; }
    virtual void CaseDesensitize();
    virtual int AddToNfa(Nfa &nfa, int startstate);
    virtual bool EqualTo(Rx *rhs);
};

class LIBGLR_EXPORT RxUnion : public Rx
{
public:
    Rx *m_lhs, *m_rhs;
    RxUnion(Rx *lhs, Rx *rhs);
    virtual ~RxUnion();
    virtual Rx *Copy();
    virtual RxType RxClassification() { return Rx::Union; }
    virtual void CaseDesensitize();
    virtual int AddToNfa(Nfa &nfa, int startstate);
    virtual bool EqualTo(Rx *rhs);
};

class LIBGLR_EXPORT RxStar : public Rx
{
public:
    Rx *m_rx;
    RxStar(Rx *rx);
    virtual ~RxStar();
    virtual Rx *Copy();
    virtual RxType RxClassification() { return Rx::Star; }
    virtual void CaseDesensitize();
    virtual int AddToNfa(Nfa &nfa, int startstate);
    virtual bool EqualTo(Rx *rhs);
};

class LIBGLR_EXPORT RxPlus : public Rx
{
public:
    Rx *m_rx;
    RxPlus(Rx *rx);
    virtual ~RxPlus();
    virtual Rx *Copy();
    virtual RxType RxClassification() { return Rx::Plus; }
    virtual void CaseDesensitize();
    virtual int AddToNfa(Nfa &nfa, int startstate);
    virtual bool EqualTo(Rx *rhs);
};

class LIBGLR_EXPORT RxOpt : public Rx
{
public:
    Rx *m_rx;
    RxOpt(Rx *rx);
    virtual ~RxOpt();
    virtual Rx *Copy();
    virtual RxType RxClassification() { return Rx::Opt; }
    virtual void CaseDesensitize();
    virtual int AddToNfa(Nfa &nfa, int startstate);
    virtual bool EqualTo(Rx *rhs);
};

class LIBGLR_EXPORT RxMany : public Rx
{
public:
    Rx *m_rx;
    int m_min, m_max;
    RxMany(Rx *rx, int min, int max);
    virtual ~RxMany();
    virtual Rx *Copy();
    virtual RxType RxClassification() { return Rx::Many; }
    virtual void CaseDesensitize();
    virtual int AddToNfa(Nfa &nfa, int startstate);
    virtual bool EqualTo(Rx *rhs);
};

class LIBGLR_EXPORT RxChars : public Rx
{
public:
    UIntSet m_chars;
    RxChars();
    RxChars(int c);
    RxChars(const UIntSet &chars);
    virtual ~RxChars();
    virtual Rx *Copy();
    virtual RxType RxClassification() { return Rx::Chars; }
    virtual void CaseDesensitize();
    virtual int AddToNfa(Nfa &nfa, int startstate);
    virtual bool EqualTo(Rx *rhs);
};

class LIBGLR_EXPORT RxCapture : public Rx
{
public:
    Rx *m_rx;
    RxCapture(Rx *rx);
    virtual ~RxCapture();
    virtual Rx *Copy();
    virtual RxType RxClassification() { return Rx::Capture; }
    virtual void CaseDesensitize();
    virtual int AddToNfa(Nfa &nfa, int startstate);
    virtual bool EqualTo(Rx *rhs);
};

Rx LIBGLR_EXPORT *RxApplyOpenClose(Rx *rx, int rxopen, int rxclose);

UIntSet LIBGLR_EXPORT LookupNamedCharSet(const char *name, bool *pbFound);

#endif
