#ifdef __cplusplus
extern "C" {
#endif
GLR_API inbuf *new_inbuf(size_t inbufsize, const char *fname, ibf_read readfnct, ibf_seek seekfnct, ibf_close closefnct)
{
  inbuf *buf = (inbuf*)vp_realloc(0, inbufsize);
  vp_memzero(buf, inbufsize);
  buf->readfnct = readfnct;
  buf->seekfnct = seekfnct;
  buf->closefnct = closefnct;
  buf->fname = fname ? fname : "";
  buf->line = buf->col = 1;
  buf->strbuf = 0;
  return buf;
}

GLR_API void free_inbuf(inbuf *buf)
{
    if( buf->closefnct ) buf->closefnct(buf);
    if(!buf->strbuf && buf->s) vp_realloc(buf->s, 0);
    vp_realloc(buf, 0);
}

GLR_API int ib_read(inbuf *buf, void *vp, int count)
{
    int rd = 0, i0, i1, n;
    char *dst = (char*)vp, *tmps = 0;
    if( ! buf->readfnct )
        return 0;
    while( count )
    {
        // Copy from the buffer if we can
        if( buf->pos < buf->count )
        {
            for( i0=0,i1=buf->pos, n=((count<=(buf->count-buf->pos))?count:(buf->count-buf->pos)); i0<n; ++i0,++i1 )
                dst[i0] = buf->s[i1];
            buf->pos += n;
            count -= n;
            rd += n;
            dst += n;
            continue;
        }
        // if we're out of file, stop.
        if( buf->eof )
            break;
        // Out of buffer, let's read large chunks directly into the destination.
        n = (count / BUF_BLOCK_SIZE) * BUF_BLOCK_SIZE;
        if(n)
        {
            buf->readfnct(buf, dst, n);
            buf->startpos += n;
            rd += n;
            dst += n;
            count -= n;
            continue;
        }
        // Out of large chunks, let's read a single chunk to the buffer (fast io, copy on next loop).
        if( buf->capacity < BUF_BLOCK_SIZE ) {
            // make sure we have a single chunk available
            tmps = (char*)vp_realloc(buf->s, BUF_BLOCK_SIZE);
            if( ! tmps ) break;
            buf->s = tmps;
            buf->capacity = BUF_BLOCK_SIZE;
        }
        buf->pos = 0;
        buf->startpos += buf->count;
        buf->count = buf->readfnct(buf, buf->s, BUF_BLOCK_SIZE);
        if( buf->count == 0 )
        {
            buf->eof = 1;
            break;
        }
    }
    return rd;
}

GLR_API foff_t ib_seek(inbuf *buf, foff_t to, int whence)
{
    // Seeks within the current buffer are handled without using seekfnct,
    // so file devices which buffer all of their contents, e.g. strings, require no seek function.
    if( whence == INBUF_SET )
    {
        if( (to>=buf->startpos) && (to<=buf->startpos+buf->count) )
        {
            buf->pos = (int)(to-buf->startpos);
            return to;
        }
    }
    else if( whence == INBUF_CUR )
    {
        if( (buf->pos+to) >= 0 && (buf->pos+to) <= buf->count )
        {
            buf->pos = buf->pos + (int)to;
            return buf->startpos+buf->pos;
        }
    }
    else if( whence == INBUF_END )
    {
        if( buf->eof && (to<=0) && (-to<=buf->count))
        {
            buf->pos = buf->count+(int)to;
            return buf->startpos+buf->pos;
        }
    }
    else
        return -1;
    // Anything with an actual backing store needs a seek funciton.
    if( ! buf->seekfnct )
        return -1;
    to = buf->seekfnct(buf, to, whence);
    if( to == -1 )
        return -1;
    buf->startpos = to;
    buf->pos = 0;
    buf->count = 0;
    return to;
}

GLR_API foff_t ib_tell(inbuf *buf)
{
    return buf->startpos + buf->pos;
}

GLR_API int read_file(fpinbuf *buf, void *dst, int count)
{
#ifdef _WIN32
  DWORD dwRead = 0;
  if( ! ReadFile(buf->h, dst, count, &dwRead, 0) )
    return 0;
  return dwRead;
#else
  ssize_t rdsz = read(buf->h, dst, count);
  if( rdsz == -1 )
      return 0;
  return rdsz;
#endif
}

GLR_API foff_t seek_file(fpinbuf *buf, foff_t to, int whence)
{
#ifdef _WIN32
    LARGE_INTEGER liDistanceToMove, liNewFilePointer;
    liDistanceToMove.QuadPart = to;
    if( ! SetFilePointerEx(buf->h, liDistanceToMove, &liNewFilePointer, whence) )
        return -1;
    return liNewFilePointer.QuadPart;
#else
    return lseek64(buf->h, to, whence);
#endif
}

GLR_API void close_file(fpinbuf *buf)
{
#ifdef _WIN32
    CloseHandle(buf->h);
#else
    close(buf->h);
#endif
}

GLR_API inbuf *h_inbuf(const char *fname, handle_t h, int handoff)
{
  fpinbuf *ret = (fpinbuf*)new_inbuf(sizeof(fpinbuf), fname, (ibf_read)read_file, (ibf_seek)seek_file, handoff ? (ibf_close)close_file : 0);
  ret->h = h;
  return (inbuf*)ret;
}

GLR_API inbuf *file_inbuf(const char *fname)
{
  fpinbuf *ret = 0;
#ifdef _WIN32
  HANDLE h = CreateFileA(fname, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if( h == INVALID_HANDLE_VALUE ) return 0;
#else
    int h = open(fname, O_RDONLY);
    if( h == -1 )
        return 0;
#endif
  return h_inbuf(fname, h, 1);
}

GLR_API int read_str(inbuf *buf, void *dst, int count) { return 0; }
GLR_API void close_str(inbuf *buf) {}

GLR_API inbuf *string_inbuf(const char *s, int staticstr)
{
  inbuf *ret = new_inbuf(sizeof(inbuf), "<string>", read_str, (ibf_seek)0, close_str);
  size_t len = 0;
  ret->strbuf = staticstr;
  if( staticstr )
      ret->eof = 1;
  if( ! staticstr )
    ret->s = nv_strdup(s, &len);
  else
  {
      ret->s = (char*)s;
      while( *s++ ) ++len;
  }
  ret->capacity = ret->count = (int)len;
  return ret;
}

GLR_API outbuf *new_ob(size_t size, obf_write writefnct, obf_flush flushfnct, obf_close closefnct)
{
    outbuf *out = (outbuf*)vp_realloc(0, size);
    vp_memzero(out, size);
    out->bufused = 0;
    out->writefnct = writefnct;
    out->flushfnct = flushfnct;
    out->closefnct = closefnct;
    out->colno = 1;
    out->lineno = 1;
    out->autoflush = 0;
    return out;
}

GLR_API void ob_write(outbuf *buf, const char *s, size_t len)
{
    int do_autoflush = 0;
    size_t a;
    for( a = 0; a < len; ++a )
    {
        if( s[a] == '\n' )
        {
            if( ! do_autoflush && buf->autoflush )
                do_autoflush = 1;
            buf->colno = 1;
            buf->lineno++;
        }
        else
            buf->colno++;
    }
    // fill the remaining buffer and write out if it is filled.
    if( buf->bufused )
    {
        a = SZ(buf->buffer)-buf->bufused;
        if( a )
        {
            if( a > len )
                a = len;
            memcpy(buf->buffer+buf->bufused, s, a);
            buf->bufused += a;
            len -= a;
            s += a;
        }
        if( buf->bufused == SZ(buf->buffer) )
        {
            buf->writefnct(buf, buf->buffer, buf->bufused);
            buf->bufused = 0;
        }
    }
    while( len >= BUF_BLOCK_SIZE )
    {
        buf->writefnct(buf, s, BUF_BLOCK_SIZE);
        len -= BUF_BLOCK_SIZE;
        s += BUF_BLOCK_SIZE;
    }
    if( len )
    {
        memcpy(buf->buffer, s, len);
        buf->bufused = len;
    }
    if( do_autoflush )
        ob_flush(buf);
}

GLR_API void ob_printf(outbuf *buf, const char *fmt, ...)
{
  va_list args;
  va_start(args,fmt);
  ob_vprintf(buf,fmt,args);
  va_end(args);
}

GLR_API void ob_vprintf(outbuf *buf, const char *fmt, va_list args)
{
  char sbuf[1024];
  size_t len = 0;
  if( ! buf )
      return;
  len = vsprintf(sbuf, fmt, args);
  if( len )
    ob_write(buf, sbuf, len);
}

GLR_API void ob_putc(outbuf *buf, int c)
{
    char ch = c;
    ob_write(buf, &ch, 1);
}

GLR_API void ob_puts(outbuf *buf, const char *s)
{
    const char *send = s;
    while( *send ) ++send;
    ob_write(buf, s, send-s);
}

GLR_API void ob_flush(outbuf *buf)
{
    if( buf->bufused )
    {
        buf->writefnct(buf, buf->buffer, buf->bufused);
        buf->bufused = 0;
    }
    if( buf->flushfnct )
        buf->flushfnct(buf);
}

GLR_API void free_ob(outbuf *buf)
{
    if( buf->bufused )
    {
        buf->writefnct(buf, buf->buffer, buf->bufused);
        buf->bufused = 0;
    }
    ob_flush(buf);
    vp_realloc(buf, 0);
}

GLR_API void file_write(file_outbuf *buf, const char *s, size_t len)
{
#ifdef _WIN32
  DWORD dwWrote = 0;
  WriteFile(buf->h, s, (DWORD)len, &dwWrote, 0);
#else
  ssize_t wrote = write(buf->h, s, len);
#endif
}

GLR_API void file_flush(file_outbuf *buf)
{
#ifdef _WIN32
    FlushFileBuffers(buf->h);
#else
    fsync(buf->h);
#endif
}

GLR_API void file_close(file_outbuf *buf)
{
#ifdef _WIN32
    CloseHandle(buf->h);
#else
    close(buf->h);
#endif
}

GLR_API outbuf *new_h_outbuf(const char *fname, handle_t h, int handoff, int autoflush)
{
    file_outbuf *out = (file_outbuf*)new_ob(sizeof(file_outbuf), (obf_write)file_write, (obf_flush)file_flush, handoff ? (obf_close)file_close : 0);
    out->h = h;
    out->base.autoflush = autoflush;
    return (outbuf*)out;
}

GLR_API outbuf *new_file_outbuf(const char *fname)
{
    file_outbuf *out = 0;
#ifdef _WIN32
    HANDLE h = CreateFileA(fname, GENERIC_WRITE, 0, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
    if( h == INVALID_HANDLE_VALUE )
        return 0;
    if( GetLastError() == ERROR_ALREADY_EXISTS )
    {
        SetFilePointer(h, 0, 0, FILE_BEGIN);
        SetEndOfFile(h);
    }
#else
    int h = open(fname, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
    if( h == -1 )
        return 0;
#endif
    return new_h_outbuf(fname, h, 1, 0);
}

GLR_API void string_write(string_outbuf *buf, const char *s, size_t len)
{
    char *dst = 0;
    if( ! len ) return;
    if( buf->count+(int)len+1 > buf->capacity )
        buf->buf = (char*)vp_realloc(buf->buf, buf->count+len+1);
    dst = buf->buf+buf->count;
    while( len-- )
        *dst++ = *s++;
    *dst = 0;
}

GLR_API void string_close(string_outbuf *buf)
{
    vp_realloc(buf->buf, 0);
}

GLR_API outbuf *new_string_outbuf()
{
    return new_ob(sizeof(string_outbuf), (obf_write)string_write, (obf_flush)0, (obf_close)string_close);
}

GLR_API void obs_clear(outbuf *buf)
{
    string_outbuf *obs = (string_outbuf*)buf;
    obs->base.bufused = 0;
    obs->count = 0;
}

GLR_API const char *obs_getstr(outbuf *buf)
{
    string_outbuf *obs = (string_outbuf*)buf;
    if( obs->base.bufused )
        ob_flush(buf);
    return obs->buf;
}

#ifdef __cplusplus
} /* extern "C" */
#endif
