// Copyright (c) 2008-2011 Mighty Heave Software LLC.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#ifndef __templatehelp_h
#define __templatehelp_h

#include "glrexport.h"
#include "ints.h"
#include <stddef.h>

// comparison typedef, and some comparisons.
typedef int (*cmpfnct)(const void *lhs, const void *rhs);
int LIBGLR_EXPORT cmpuint32(const uint32_t *lhs, const uint32_t *rhs);

// Binary search.
bool LIBGLR_EXPORT find_binsearch(void *base, size_t num, size_t size, cmpfnct cmp, void *test, int *index);

// Binary search, with duplicates allowed.
bool LIBGLR_EXPORT findrange_binsearch(void *base, size_t num, size_t size, cmpfnct cmp, void *test, int *begin, int *end);

// A pretty good sort function.
void LIBGLR_EXPORT glr_quicksort(void *data, size_t datasize, size_t count, cmpfnct cmp);

// A pretty good string hash function.
uint32_t LIBGLR_EXPORT hash_djb2(const char *str);

// An ok integer hash function
uint32_t LIBGLR_EXPORT hash_jenkins(uint32_t a);

// Bob Jenkins hashbig function.
uint32_t LIBGLR_EXPORT hash_uint32s(const uint32_t *values, size_t count, uint32_t initval);

// A pretty good reverse function.
void LIBGLR_EXPORT glr_reverse(void *data, int datasize, int count);

#ifdef _WIN32
// round to the nearest int, return a double.
double LIBGLR_EXPORT round(double d);
#endif

template<class T>
int simplecmp(const T *lhs, const T *rhs)
{
    if( *lhs == *rhs )
	    return 0;
    if( *lhs < *rhs )
	    return -1;
    if( *rhs < *lhs )
	    return 1;
    // ?
    return 1;
}

template<class T>
class simpleeq
{
public:
    bool operator()(const T &lhs, const T &rhs) const
    {
        return lhs == rhs;
    }
};

template<class T>
class simplehash
{
public:
    uint32_t operator()(const T &t) const
    {
        return hash_jenkins((uint32_t)(uint64_t)t);
    }
};

#endif /* __templatehelp_h */
